/* BDF - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi.jsonproducers;

import fr.exemole.bdfserver.multi.Multi;
import fr.exemole.bdfserver.multi.MultiConstants;
import fr.exemole.bdfserver.multi.MultiUtils;
import fr.exemole.bdfserver.tools.synthesis.RedacteurSynthesis;
import fr.exemole.bdfserver.tools.synthesis.SphereSynthesis;
import java.text.ParseException;
import net.fichotheque.SubsetKey;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class MultiJsonProducerFactory {

    private MultiJsonProducerFactory() {

    }

    public static MultiJsonProducer getJsonProducer(Multi multi, String json, CommandMessage commandMessage, RequestMap requestMap) {
        switch (json) {
            case MultiConstants.FICHOTHEQUE_ARRAY_JSON: {
                MultiJsonProducer multiJsonProducer = new FichothequeArrayJsonProducer(multi);
                multiJsonProducer.setCommandMessage(commandMessage);
                return multiJsonProducer;
            }
            case MultiConstants.PERSON_JSON: {
                try {
                    MultiJsonProducer multiJsonProducer = getPersonJsonProducer(multi, requestMap);
                    if (commandMessage != null) {
                        multiJsonProducer.setCommandMessage(commandMessage);
                    }
                    return multiJsonProducer;
                } catch (ErrorMessageException eme) {
                    return getPingJsonProducer(multi, eme.getErrorMessage());
                }
            }
            case MultiConstants.PING_JSON: {
                return getPingJsonProducer(multi, commandMessage);
            }
            default: {
                if (commandMessage == null) {
                    commandMessage = LogUtils.error("_ error.unknown.json", json);
                }
                return getPingJsonProducer(multi, commandMessage);
            }

        }
    }

    private static MultiJsonProducer getPersonJsonProducer(Multi multi, RequestMap requestMap) throws ErrorMessageException {
        if (!multi.isAuthentificationSharingAllowed()) {
            throw new ErrorMessageException("_ error.unsupported.sharingnotallowed");
        }
        String sphereName = requestMap.getParameter("sphere");
        if (sphereName == null) {
            throw new ErrorMessageException("_ error.empty.mandatoryparameter", "sphere");
        }
        String login = requestMap.getParameter("login");
        if (login == null) {
            throw new ErrorMessageException("_ error.empty.mandatoryparameter", "login");
        }
        SubsetKey sphereKey;
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName.trim());
        } catch (ParseException pe) {
            throw new ErrorMessageException("_ error.unknown.sphere", sphereName);
        }
        SphereSynthesis sphereSynthesis = MultiUtils.getSphereSynthesis(multi, sphereKey);
        if (sphereSynthesis.getFichothequeSize() == 0) {
            throw new ErrorMessageException("_ error.unknown.sphere", sphereName);
        }
        RedacteurSynthesis redacteurSynthesis = sphereSynthesis.getRedacteurSynthesis(login);
        if (redacteurSynthesis == null) {
            throw new ErrorMessageException("_ error.unknown.redacteur", login);
        }
        return new PersonJsonProducer(multi, sphereSynthesis, redacteurSynthesis, multi.getMultiConf().getLang());
    }

    private static MultiJsonProducer getPingJsonProducer(Multi multi, CommandMessage commandMessage) {
        MultiJsonProducer multiJsonProducer = new PingJsonProducer(multi, true);
        multiJsonProducer.setCommandMessage(commandMessage);
        return multiJsonProducer;
    }

}
