/* BdfServer - Copyright (c) 2006-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.impl;

import fr.exemole.bdfserver.api.BdfExtensionInitializer;
import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.EditSessionListener;
import fr.exemole.bdfserver.api.configuration.LangConfiguration;
import fr.exemole.bdfserver.api.managers.BalayageManager;
import fr.exemole.bdfserver.api.managers.ExtensionManager;
import fr.exemole.bdfserver.api.managers.ExternalScriptManager;
import fr.exemole.bdfserver.api.managers.GroupManager;
import fr.exemole.bdfserver.api.managers.L10nManager;
import fr.exemole.bdfserver.api.managers.PasswordManager;
import fr.exemole.bdfserver.api.managers.PermissionManager;
import fr.exemole.bdfserver.api.managers.PolicyManager;
import fr.exemole.bdfserver.api.managers.ScrutariExportManager;
import fr.exemole.bdfserver.api.managers.SqlExportManager;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.managers.TreeManager;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.storage.BdfUserStorage;
import fr.exemole.bdfserver.api.storage.JsAnalyser;
import fr.exemole.bdfserver.api.storage.MigrationEngine;
import fr.exemole.bdfserver.api.storage.ResourceStorage;
import fr.exemole.bdfserver.api.storage.StorageCheck;
import fr.exemole.bdfserver.api.storage.StorageRoot;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.conf.BdfServerDirs;
import fr.exemole.bdfserver.tools.externalsource.CoreExternalSourceProvider;
import fr.exemole.bdfserver.tools.storage.ResourceJsAnalyser;
import fr.exemole.bdfserver.tools.storage.StorageCollection;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.List;
import java.util.Map;
import net.fichotheque.EditOrigin;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.FichothequeEditorProvider;
import net.fichotheque.exportation.table.TableExportContext;
import net.fichotheque.externalsource.ExternalSourceProvider;
import net.fichotheque.format.FormatContext;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.metadata.ThesaurusLangChecker;
import net.mapeadores.util.buildinfo.BuildInfo;
import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.css.SacParserInit;
import net.mapeadores.util.logging.MessageLog;
import net.mapeadores.util.mimetype.MimeTypeResolver;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.fichotheque.selection.FicheQuery;


/**
 *
 * @author Vincent Calame
 */
public class BdfServerImpl implements BdfServer {

    static {
        SacParserInit.init();
    }

    private final String name;
    private final Map<String, Object> contextObjectMap;
    private final BdfServerDirs bdfServerDirs;
    private final Fichotheque fichotheque;
    private final FichothequeEditorProvider fichothequeEditorProvider;
    private final PermissionManagerImpl permissionManager;
    private final GroupManagerImpl groupManager;
    private final TreeManagerImpl treeManager;
    private final PasswordManagerImpl passwordManager;
    private final UiManagerImpl uiManager;
    private final PolicyManagerImpl policyManager;
    private final BdfServerFichothequeListener fichothequeListener;
    private final BalayageManagerImpl balayageManager;
    private final ScrutariExportManagerImpl scrutariExportManager;
    private final SqlExportManagerImpl sqlExportManager;
    private final LangConfigurationImpl langConfiguration;
    private final L10nManagerImpl l10nManager;
    private final ExtensionManagerImpl extensionManager;
    private final TransformationManagerImpl transformationManager;
    private final FormatContextImpl formatContext;
    private final TableExportContextImpl tableExportContext;
    private final TableExportManagerImpl tableExportManager;
    private final ExternalScriptManagerImpl externalScriptManager;
    private final ThesaurusLangCheckerImpl thesaurusLangChecker;
    private final JsAnalyser jsAnalyser;
    private final MessageLog initMessageLog;
    private final BuildInfo buildInfo;
    private final List<ResourceStorage> resourceStorageList;
    private final StorageRoot outputStorage;
    private final StorageRoot cacheStorage;
    private final MimeTypeResolver mimeTypeResolver;
    private final BdfUserStorage bdfUserStorage;
    private final ExternalSourceProvider externalSourceProvider;
    private final short initState;
    private EditSessionListener[] editSessionListenerArray = new EditSessionListener[0];


    public BdfServerImpl(String name, StorageCollection storageCollection, BuildInfo buildInfo, List<BdfExtensionInitializer> extensionInitializerList, Map<String, Object> contextObjectMap) {
        this.name = name;
        this.contextObjectMap = contextObjectMap;
        this.bdfServerDirs = storageCollection.getBdfServerDirs();
        this.buildInfo = buildInfo;
        this.mimeTypeResolver = MimeTypeUtils.DEFAULT_RESOLVER;
        this.initMessageLog = storageCollection.getInitMessageLog();
        StorageCheck storageCheck = storageCollection.getStorageCheck();
        this.bdfUserStorage = storageCollection.getBdfUserStorage();
        this.resourceStorageList = storageCollection.getResourceStorageList();
        this.outputStorage = storageCollection.getOutputStorage();
        this.cacheStorage = storageCollection.getCacheStorage();
        this.fichothequeEditorProvider = storageCollection.getFichothequeEditorProvider();
        this.fichotheque = fichothequeEditorProvider.getFichotheque();
        this.langConfiguration = new LangConfigurationImpl(storageCollection.getConfigurationStorage());
        langConfiguration.update(storageCheck.getLangConfiguration());
        this.permissionManager = PermissionManagerImpl.build(fichotheque, storageCollection.getRoleStorage(), storageCollection.getRedacteurList(), storageCheck);
        this.groupManager = GroupManagerImpl.build(storageCollection.getGroupStorage(), storageCheck.getGroupDefList());
        this.treeManager = TreeManagerImpl.build(fichotheque, storageCollection.getTreeStorage(), storageCheck.getSubsetTreeMap());
        this.passwordManager = new PasswordManagerImpl(fichotheque, storageCollection.getPasswordStorage());
        this.policyManager = PolicyManagerImpl.build(fichotheque, storageCollection.getPolicyStorage(), storageCheck);
        this.scrutariExportManager = new ScrutariExportManagerImpl(storageCollection.getScrutariExportStorage());
        scrutariExportManager.check(storageCheck.getScrutariExportDefList());
        this.sqlExportManager = new SqlExportManagerImpl(storageCollection.getSqlExportStorage());
        sqlExportManager.check(storageCheck.getSqlExportDefList());
        this.l10nManager = new L10nManagerImpl(resourceStorageList);
        this.jsAnalyser = new ResourceJsAnalyser(resourceStorageList, mimeTypeResolver);
        this.externalScriptManager = new ExternalScriptManagerImpl(this);
        this.extensionManager = new ExtensionManagerImpl(fichotheque, storageCollection.getBdfExtensionStorage(), extensionInitializerList, storageCollection.getConfigurationStorage());
        extensionManager.update(storageCheck.getActiveExtensionList());
        this.thesaurusLangChecker = new ThesaurusLangCheckerImpl();
        this.balayageManager = new BalayageManagerImpl(fichotheque, storageCollection.getBalayageStorage());
        this.uiManager = UiManagerImpl.build(this, storageCollection.getUiStorage(), storageCheck.getUiCheckMap());
        this.transformationManager = new TransformationManagerImpl(this, storageCollection.getTemplateStorage());
        this.formatContext = new FormatContextImpl(this, extensionManager.getInstructionResolverProvider(), transformationManager, l10nManager, uiManager);
        this.tableExportContext = new TableExportContextImpl(fichotheque, extensionManager, formatContext, langConfiguration, l10nManager, uiManager);
        this.tableExportManager = new TableExportManagerImpl(fichotheque, storageCollection.getTableExportStorage(), tableExportContext);
        this.externalSourceProvider = new CoreExternalSourceProvider(this);
        this.fichothequeListener = new BdfServerFichothequeListener(this);
        fichotheque.addFichothequeListener(fichothequeListener);
        fichotheque.addFichothequeListener(treeManager);
        fichotheque.addFichothequeListener(uiManager);
        extensionManager.addEditionListeners(this);
        MigrationEngine migrationEngine = storageCollection.getMigrationEngine();
        if (migrationEngine != null) {
            migrationEngine.afterInitRun(this);
        }
        boolean okState = false;
        try {
            BdfUser adminBdfUser = BdfUserUtils.getFirstAdminBdfUser(this);
            this.transformationManager.update();
            this.tableExportManager.update();
            this.balayageManager.update();
            okState = true;
        } catch (IllegalStateException ise) {

        }
        if (okState) {
            this.initState = BdfServer.OK_STATE;
        } else {
            this.initState = BdfServer.NOADMIN_STATE;
        }
    }

    @Override
    public short getInitState() {
        return initState;
    }

    @Override
    public MessageLog getInitMessageLog() {
        return initMessageLog;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public Object getContextObject(String objectName) {
        if (objectName.equals(BdfServerConstants.BDFSERVER_CONTEXTOBJECT)) {
            return this;
        } else {
            return contextObjectMap.get(objectName);
        }
    }

    @Override
    public BdfServerDirs getBdfServerDirs() {
        return bdfServerDirs;
    }

    @Override
    public Fichotheque getFichotheque() {
        return fichotheque;
    }

    @Override
    public PermissionManager getPermissionManager() {
        return permissionManager;
    }

    @Override
    public PasswordManager getPasswordManager() {
        return passwordManager;
    }

    @Override
    public TransformationManager getTransformationManager() {
        return transformationManager;
    }

    @Override
    public UiManager getUiManager() {
        return uiManager;
    }

    @Override
    public PolicyManager getPolicyManager() {
        return policyManager;
    }

    @Override
    public TableExportManager getTableExportManager() {
        return tableExportManager;
    }

    @Override
    public ScrutariExportManager getScrutariExportManager() {
        return scrutariExportManager;
    }

    @Override
    public SqlExportManager getSqlExportManager() {
        return sqlExportManager;
    }

    @Override
    public ExtensionManager getExtensionManager() {
        return extensionManager;
    }

    @Override
    public BalayageManager getBalayageManager() {
        return balayageManager;
    }

    @Override
    public ExternalScriptManager getExternalScriptManager() {
        return externalScriptManager;
    }

    @Override
    public BdfUserStorage getBdfUserStorage() {
        return bdfUserStorage;
    }

    @Override
    public BdfServerEditor getBdfServerEditor() {
        return new BdfServerEditorImpl(this);
    }

    @Override
    public GroupManager getGroupManager() {
        return groupManager;
    }

    @Override
    public TreeManager getTreeManager() {
        return treeManager;
    }

    @Override
    public L10nManager getL10nManager() {
        return l10nManager;
    }

    @Override
    public BdfUser createBdfUser(Redacteur redacteur) {
        BdfUserImpl bdfUser = new BdfUserImpl(redacteur);
        bdfUser.init(this);
        return bdfUser;
    }

    @Override
    public void updateFicheSelection(BdfUser bdfUser) {
        ((BdfUserImpl) bdfUser).updateFicheSelection(this);
    }

    @Override
    public void setFicheSelection(BdfUser bdfUser, FicheQuery ficheQuery, String sortType) {
        ((BdfUserImpl) bdfUser).setFicheSelection(this, ficheQuery, sortType);
    }

    @Override
    public LangConfiguration getLangConfiguration() {
        return langConfiguration;
    }

    @Override
    public void bufferFichothequeListener() {
        fichothequeListener.buffer();
    }

    @Override
    public void flushFichothequeListener() {
        fichothequeListener.flush();
    }

    @Override
    public FichothequeEditorProvider getFichothequeEditorProvider() {
        return fichothequeEditorProvider;
    }

    @Override
    public void addEditionSessionListener(EditSessionListener editSessionListener) {
        int length = editSessionListenerArray.length;
        editSessionListenerArray = (EditSessionListener[]) ArrayUtils.addUnique(editSessionListenerArray, editSessionListener, new EditSessionListener[length + 1]);
    }

    @Override
    public void removeEditionSessionListener(EditSessionListener editSessionListener) {
        int length = editSessionListenerArray.length;
        if (length > 0) {
            editSessionListenerArray = (EditSessionListener[]) ArrayUtils.removeUnique(editSessionListenerArray, editSessionListener, new EditSessionListener[length - 1]);
        }
    }

    @Override
    public EditSession initEditSession(EditOrigin editOrigin) {
        return new InternalEditSession(this, fichothequeEditorProvider.newFichothequeEditor(editOrigin));
    }

    @Override
    public void closeEditSession(EditSession editSession) {
        editSession.getFichothequeEditor().saveChanges();
    }

    @Override
    public FormatContext getFormatContext() {
        return formatContext;
    }

    @Override
    public BuildInfo getBuildInfo() {
        return buildInfo;
    }

    @Override
    public List<ResourceStorage> getResourceStorageList() {
        return resourceStorageList;
    }

    @Override
    public StorageRoot getOutputStorage() {
        return outputStorage;
    }

    @Override
    public StorageRoot getCacheStorage() {
        return cacheStorage;
    }

    @Override
    public MimeTypeResolver getMimeTypeResolver() {
        return mimeTypeResolver;
    }

    @Override
    public JsAnalyser getJsAnalyser() {
        return jsAnalyser;
    }

    @Override
    public ExternalSourceProvider getExternalSourceProvider() {
        return externalSourceProvider;
    }

    @Override
    public ThesaurusLangChecker getThesaurusLangChecker() {
        return thesaurusLangChecker;
    }

    @Override
    public TableExportContext getTableExportContext() {
        return tableExportContext;
    }


    private void fireUseOfFichothequeEditor() {
        EditSessionListener[] array = editSessionListenerArray;
        int length = array.length;
        for (int i = 0; i < length; i++) {
            array[i].useOfFichothequeEditor();
        }
    }


    private static class InternalEditSession implements EditSession {

        private final BdfServerImpl bdfServer;
        private final FichothequeEditor fichothequeEditor;

        private InternalEditSession(BdfServerImpl bdfServer, FichothequeEditor fichothequeEditor) {
            this.bdfServer = bdfServer;
            this.fichothequeEditor = fichothequeEditor;
        }

        @Override
        public BdfServer getBdfServer() {
            return bdfServer;
        }

        @Override
        public FichothequeEditor getFichothequeEditor() {
            bdfServer.fireUseOfFichothequeEditor();
            return fichothequeEditor;
        }

    }

}
