/* BdfServer - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.impl;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.L10nManager;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import java.text.ParseException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.format.FormatContext;
import net.fichotheque.format.FormatSource;
import net.fichotheque.format.formatters.ExtractionFormatter;
import net.fichotheque.format.formatters.FicheBlockFormatter;
import net.fichotheque.include.LiageTest;
import net.fichotheque.tools.format.formatters.ExtractionFormatterParser;
import net.fichotheque.xml.extraction.ExtractionXMLUtils;
import net.mapeadores.util.instruction.InstructionResolverProvider;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.MessageLocalisationProvider;


/**
 *
 * @author Vincent Calame
 */
class FormatContextImpl implements FormatContext {

    private final static Map<String, Object> EMPTY_PARAMMAP = Collections.emptyMap();
    private final BdfServer bdfServer;
    private final InstructionResolverProvider instructionResolverProvider;
    private final TransformationManager transformationManager;
    private final L10nManager l10nManager;
    private final UiManager uiManager;

    FormatContextImpl(BdfServer bdfServer, InstructionResolverProvider instructionResolverProvider, TransformationManager transformationManager, L10nManager l10nManager, UiManager uiManager) {
        this.bdfServer = bdfServer;
        this.instructionResolverProvider = instructionResolverProvider;
        this.transformationManager = transformationManager;
        this.l10nManager = l10nManager;
        this.uiManager = uiManager;
    }

    @Override
    public Fichotheque getFichotheque() {
        return bdfServer.getFichotheque();
    }

    @Override
    public InstructionResolverProvider getInstructionResolverProvider() {
        return instructionResolverProvider;
    }

    @Override
    public ExtractionFormatter getExtractionFormatter(String name, Map<String, Object> options) {
        if ((name == null) || (name.equals(FormatContext.XML_SPECIAL_FORMATTER_NAME))) {
            return ExtractionFormatterParser.XML_PART;
        }
        try {
            TemplateKey templateKey = TemplateKey.parse(TransformationKey.FORMAT_TRANSFORMATIONKEY, name);
            if (!transformationManager.containsTemplate(templateKey)) {
                return null;
            }
            SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
            return new SimpleTemplateExtractionFormatter(simpleTemplate, convertToParamMap(options));
        } catch (ParseException pe) {
            return null;
        }
    }

    @Override
    public FicheBlockFormatter getFicheBlockFormatter(String name, Map<String, Object> options) {
        if ((name != null) && (name.equals(FormatContext.XML_SPECIAL_FORMATTER_NAME))) {
            return new SimpleTemplateFicheBlockFormatter(null, EMPTY_PARAMMAP);
        }
        try {
            TemplateKey templateKey = TemplateKey.parse(TransformationKey.SECTION_TRANSFORMATIONKEY, name);
            if (!transformationManager.containsTemplate(templateKey)) {
                return null;
            }
            SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
            return new SimpleTemplateFicheBlockFormatter(simpleTemplate, convertToParamMap(options));
        } catch (ParseException pe) {
            return null;
        }
    }

    @Override
    public LiageTest getLiageTest(Corpus corpus) {
        return UiUtils.checkLiageTest(uiManager.getMainUiComponents(corpus));
    }

    @Override
    public MessageLocalisationProvider getMessageLocalisationProvider() {
        return l10nManager;
    }

    @Override
    public Object getContextObject(String objectName) {
        return bdfServer.getContextObject(objectName);
    }

    private static Map<String, Object> newParamMap(Lang lang, Map<String, Object> paramMap) {
        Map<String, Object> newParamMap = BdfTransformationUtils.newParamMap(lang);
        newParamMap.putAll(paramMap);
        return newParamMap;
    }

    private static Map<String, Object> convertToParamMap(Map<String, Object> options) {
        if (options.isEmpty()) {
            return EMPTY_PARAMMAP;
        }
        Map<String, Object> result = new HashMap<String, Object>();
        for (Map.Entry<String, Object> entry : options.entrySet()) {
            Object value = entry.getValue();
            switch (entry.getKey()) {
                case "startlevel":
                    if (value instanceof Integer) {
                        result.put(TransformationConstants.STARTLEVEL_PARAMETER, value);
                    }
                    break;
                case "externaltarget":
                    result.put(TransformationConstants.EXTERNALTARGET_PARAMETER, value);
                    break;
            }
        }
        return result;
    }


    private static class SimpleTemplateExtractionFormatter implements ExtractionFormatter {

        private final SimpleTemplate simpleTemplate;
        private final Map<String, Object> paramMap;

        private SimpleTemplateExtractionFormatter(SimpleTemplate simpleTemplate, Map<String, Object> paramMap) {
            this.simpleTemplate = simpleTemplate;
            this.paramMap = paramMap;
        }

        @Override
        public String formatExtraction(String extractionString, FormatSource formatSource) {
            Lang lang = LocalisationUtils.getDefaultLang(formatSource.getLangContext());
            Map<String, Object> newParamMap = newParamMap(lang, paramMap);
            return simpleTemplate.transform(extractionString, newParamMap);
        }

    }


    private static class SimpleTemplateFicheBlockFormatter implements FicheBlockFormatter {

        private final SimpleTemplate simpleTemplate;
        private final Map<String, Object> paramMap;

        private SimpleTemplateFicheBlockFormatter(SimpleTemplate simpleTemplate, Map<String, Object> paramMap) {
            this.simpleTemplate = simpleTemplate;
            this.paramMap = paramMap;
        }

        @Override
        public String formatFicheBlocks(FicheBlocks ficheBlocks, FormatSource formatSource, SubsetKey defaultCorpusKey) {
            String extractionString = ExtractionXMLUtils.ficheBlocksToExtractionString(ficheBlocks, formatSource.getExtractionContext(), defaultCorpusKey);
            if (simpleTemplate == null) {
                return extractionString;
            } else {
                Lang defaultLang = LocalisationUtils.getDefaultLang(formatSource.getLangContext());
                Map<String, Object> newParamMap = newParamMap(defaultLang, paramMap);
                return simpleTemplate.transform(extractionString, newParamMap);
            }
        }

    }

}
