/* BdfServer - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.LangConfiguration;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.exportation.transformation.SimpleTemplateFactory;
import fr.exemole.bdfserver.api.exportation.transformation.StreamTemplateFactory;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.api.storage.TemplateStorage.Unit;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.BdfURI;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.CompilationException;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.SimpleTemplateCompiler;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.TemplateCompiler;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.TemplateCompilerUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.XsltOdStreamTemplateCompiler;
import fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates.TemplateStorageUnitFactory;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.function.Predicate;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.LiageTest;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.exportation.transformation.TransformationCheck;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.tools.extraction.builders.DocumentFilterBuilder;
import net.fichotheque.tools.extraction.builders.ExtractionDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitAddendaDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitAlbumDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitCorpusDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtraitThesaurusDefBuilder;
import net.fichotheque.tools.extraction.builders.FicheFilterBuilder;
import net.fichotheque.tools.extraction.builders.FicheRattachementDefBuilder;
import net.fichotheque.tools.extraction.builders.FilterUnitListBuilder;
import net.fichotheque.tools.extraction.builders.IllustrationFilterBuilder;
import net.fichotheque.tools.extraction.builders.IntituleClauseBuilder;
import net.fichotheque.tools.extraction.builders.MotcleFilterBuilder;
import net.fichotheque.tools.extraction.builders.MotcleRattachementDefBuilder;
import net.fichotheque.tools.selection.DocumentQueryBuilder;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.tools.selection.IllustrationQueryBuilder;
import net.fichotheque.tools.selection.MotcleQueryBuilder;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.fichotheque.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.MessageByLine;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public final class BdfTransformationUtils {

    private final static Set<String> DEFAULT_SET = new HashSet<String>();
    private final static Set<String> EMPTY_SET = Collections.emptySet();
    private final static TemplateDescription[] EMPTY_ARRAY = new TemplateDescription[0];


    static {
        DEFAULT_SET.add("odt");
    }

    private BdfTransformationUtils() {
    }

    public static TemplateDescription getTemplateDescription(TransformationManager transformationManager, TemplateKey templateKey) {
        TransformationDescription transformationDescription = transformationManager.getTransformationDescription(templateKey.getTransformationKey());
        if (templateKey.isSimpleTemplate()) {
            for (TemplateDescription templateDescription : transformationDescription.getSimpleTemplateDescriptionList()) {
                if (templateDescription.getTemplateKey().equals(templateKey)) {
                    return templateDescription;
                }
            }
        } else {
            for (TemplateDescription templateDescription : transformationDescription.getStreamTemplateDescriptionList()) {
                if (templateDescription.getTemplateKey().equals(templateKey)) {
                    return templateDescription;
                }
            }
        }
        return null;
    }


    public static String getFicheString(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, String simpleTemplateName, FicheMeta ficheMeta, Lang lang, String fichothequePath) {
        LangContext langContext = LocalisationUtils.toUserLangContext(lang);
        TransformationKey corpusTransformationKey = new TransformationKey(ficheMeta.getCorpus().getSubsetKey());
        TemplateKey templateKey;
        try {
            templateKey = TemplateKey.parse(corpusTransformationKey, simpleTemplateName);
        } catch (ParseException pe) {
            templateKey = TemplateKey.toDefault(corpusTransformationKey);
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
        if (simpleTemplate == null) {
            simpleTemplate = transformationManager.getSimpleTemplate(templateKey.toDefaultTemplateKey());
        }
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, langContext)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        ExtractionDef extractionDef = simpleTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, corpusTransformationKey);
        }
        String extractionString = ExtractionXMLPart.toString(extractionContext, extractionDef, BuilderUtils.getExtractionSource(ficheMeta, extractionContext, extractionDef), false);
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(lang);
        paramMap.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, fichothequePath);
        return simpleTemplate.transform(extractionString, paramMap);
    }

    public static boolean isDefaultStreamTemplateAvailable(TransformationManager transformationManager, TransformationKey transformationKey, String extension) {
        Set<String> set = getDefaultStreamTemplateExtensionSet(transformationKey);
        if (set != null) {
            return set.contains(extension);
        } else {
            return false;
        }
    }

    public static Map<String, Object> newParamMap(Lang workingLang) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put(TransformationConstants.WORKINGLANG_PARAMETER, workingLang.toString());
        return map;
    }

    public static Map<String, Object> newParamMap(BdfServer bdfServer, BdfUser bdfUser) {
        Redacteur redacteur = bdfUser.getRedacteur();
        Map<String, Object> map = new HashMap<String, Object>();
        map.put(TransformationConstants.WORKINGLANG_PARAMETER, bdfUser.getWorkingLang().toString());
        map.put(TransformationConstants.BDF_VERSION_PARAMETER, bdfServer.getBuildInfo().getVersion());
        Attribute attribute = bdfServer.getFichotheque().getFichothequeMetadata().getAttributes().getAttribute(BdfUserSpace.INCLUDESCRIPTS_KEY);
        String includeScripts = "";
        if (attribute != null) {
            int size = attribute.size();
            if (size == 1) {
                includeScripts = attribute.getFirstValue();
            } else {
                StringBuilder buf = new StringBuilder();
                for (String value : attribute) {
                    if (buf.length() > 0) {
                        buf.append(' ');
                    }
                    buf.append(value);
                }
                includeScripts = buf.toString();
            }
        }
        map.put(TransformationConstants.INCLUDESCRIPTS_PARAMETER, includeScripts);
        map.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, BdfUserUtils.getRootUrl(bdfUser));
        boolean hideEmpty = BdfUserUtils.getBoolean(bdfUser, BdfUserSpace.HIDEEMPTY_KEY);
        map.put(TransformationConstants.SHOWEMPTY_PARAMETER, (hideEmpty) ? 0 : 1);
        map.put(TransformationConstants.USER_SPHERE_PARAMETER, redacteur.getSphere().getSubsetKey().getSubsetName());
        map.put(TransformationConstants.USER_LOGIN_PARAMETER, redacteur.getLogin());
        return map;
    }

    public static void putFileNameParameter(Map<String, String> paramMap, String fileName) {
        paramMap.put(TransformationConstants.BDF_FILENAME_PARAMETER, fileName);
        String fileBaseName = fileName;
        String fileExtension = "";
        int idx = fileName.lastIndexOf('.');
        if (idx != -1) {
            fileBaseName = fileName.substring(0, idx);
            fileExtension = fileName.substring(idx + 1);
        }
        paramMap.put(TransformationConstants.BDF_FILEBASENAME_PARAMETER, fileBaseName);
        paramMap.put(TransformationConstants.BDF_FILEEXTENSION_PARAMETER, fileExtension);
    }

    public static TemplateStorage.Unit getDefaultUnit(BdfServer bdfServer, TransformationKey transformationKey, String extension) {
        if (extension == null) {
            return TemplateStorageUnitFactory.newXsltHtml(bdfServer, TemplateKey.toDefault(transformationKey), true, false);
        } else if (isDefaultStreamTemplateAvailable(bdfServer.getTransformationManager(), transformationKey, extension)) {
            try {
                ValidExtension validExtension = ValidExtension.parse(extension);
                TemplateKey templateKey = TemplateKey.toDefault(transformationKey, validExtension);
                if (extension.equals("odt")) {
                    return TemplateStorageUnitFactory.newXsltOdt(bdfServer, templateKey, false);
                } else {
                    return null;
                }
            } catch (ParseException pe) {
                return null;
            }
        } else {
            return null;
        }
    }

    public static SimpleTemplateFactory getDefaultSimpleTemplateFactory(BdfServer bdfServer, PathConfiguration pathConfiguration, TransformationKey transformationKey) {
        TemplateKey templateKey = TemplateKey.toDefault(transformationKey);
        Unit templateStorageUnit = TemplateStorageUnitFactory.newXsltHtml(bdfServer, templateKey, true, false);
        SimpleTemplateCompiler simpleTemplateCompiler = TemplateCompilerUtils.getSimpleTemplateCompiler("xslt", bdfServer, pathConfiguration, templateKey, new ShouldNotOccurMessageHandler());
        try {
            return simpleTemplateCompiler.compile(templateStorageUnit);
        } catch (CompilationException ce) {
            throw new ShouldNotOccurException(getShouldNotOccurMessage(simpleTemplateCompiler));
        }
    }

    public static StreamTemplateFactory getDefaultStreamTemplateFactory(BdfServer bdfServer, PathConfiguration pathConfiguration, TransformationKey transformationKey, String extension) {
        ValidExtension validExtension;
        try {
            validExtension = ValidExtension.parse(extension);
        } catch (ParseException pe) {
            return null;
        }
        TemplateKey templateKey = TemplateKey.toDefault(transformationKey, validExtension);
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        if (isDefaultStreamTemplateAvailable(transformationManager, transformationKey, extension)) {
            if (extension.equals("odt")) {
                Unit templateStorageUnit = TemplateStorageUnitFactory.newXsltOdt(bdfServer, templateKey, false);
                XsltOdStreamTemplateCompiler compiler = new XsltOdStreamTemplateCompiler(bdfServer, pathConfiguration, templateKey, new ShouldNotOccurMessageHandler());
                try {
                    return compiler.compile(templateStorageUnit);
                } catch (CompilationException ce) {
                    throw new ShouldNotOccurException(getShouldNotOccurMessage(compiler));
                }
            } else {
                throw new SwitchException("extension = " + extension);
            }
        } else {
            return null;
        }
    }

    public static ExtractionDef getDefaultExtractionDef(BdfServer bdfServer, TransformationKey transformationKey) {
        ExtractionDef defaultExtractionDef;
        if (transformationKey.isCorpusTransformationKey()) {
            defaultExtractionDef = getFicheExtractionDef(bdfServer, transformationKey.toCorpusKey());
        } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            defaultExtractionDef = getCompilationExtractionDef(bdfServer);
        } else {
            if (transformationKey.equals(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY)) {
                MotcleFilter thesaurusFilter = MotcleFilterBuilder.init()
                        .setWithLabels(true)
                        .addDefaultIndexationExtraitCorpusDef()
                        .setChildrenRecursive()
                        .setWithThesaurusTitle(true)
                        .toMotcleFilter();
                defaultExtractionDef = ExtractionDefBuilder.init()
                        .setDynamicExtraitThesaurusDef(BuilderUtils.getExtraitThesaurusDef(thesaurusFilter))
                        .toExtractionDef();
            } else {
                defaultExtractionDef = null;
            }
        }
        return defaultExtractionDef;
    }


    public static Transformer newTransformer(BdfServer bdfServer, PathConfiguration pathConfiguration, Templates templates) {
        try {
            Transformer transformer = templates.newTransformer();
            transformer.setURIResolver(wrap(BdfURI.getURIResolver(bdfServer, pathConfiguration)));
            return transformer;
        } catch (TransformerException e) {
            throw new NestedTransformerException(e);
        }
    }

    public static URIResolver wrap(URIResolver uriResolver) {
        return new InternalURIResolver(uriResolver);
    }

    public static TransformerFactory newTransformationFactory(BdfServer bdfServer, PathConfiguration pathConfiguration) {
        TransformerFactory factory = TransformerFactory.newInstance();
        factory.setURIResolver(wrap(BdfURI.getURIResolver(bdfServer, pathConfiguration)));
        return factory;
    }

    public static ExtensionInfo[] getExtensionInfoArray(TransformationCheck transformationCheck) {
        SortedMap<String, ExtensionInfo> resultMap = new TreeMap<String, ExtensionInfo>();
        Set<String> withDefaultSet = getDefaultStreamTemplateExtensionSet(transformationCheck.getTransformationKey());
        for (String extension : withDefaultSet) {
            TemplateDescription[] templateDescriptionArray = transformationCheck.getStreamTemplateDescriptionArray(extension);
            if (templateDescriptionArray == null) {
                templateDescriptionArray = EMPTY_ARRAY;
            }
            resultMap.put(extension, new ExtensionInfo(extension, getAction(extension), LocalisationUtils.toMessage("_ link.global.version", extension.toUpperCase()), true, templateDescriptionArray));
        }
        for (String availableExtension : transformationCheck.getExtensionArray()) {
            if (!resultMap.containsKey(availableExtension)) {
                TemplateDescription[] templateDescriptionArray = transformationCheck.getStreamTemplateDescriptionArray(availableExtension);
                resultMap.put(availableExtension, new ExtensionInfo(availableExtension, getAction(availableExtension), LocalisationUtils.toMessage("_ link.global.version", availableExtension.toUpperCase()), false, templateDescriptionArray));
            }
        }
        return resultMap.values().toArray(new ExtensionInfo[resultMap.size()]);
    }


    public static boolean isUniquelang(BdfServer bdfServer) {
        LangConfiguration langConfiguration = bdfServer.getLangConfiguration();
        if (langConfiguration.isAllLanguages()) {
            return false;
        }
        int count = langConfiguration.getWorkingLangs().size();
        if ((count == 1) && (langConfiguration.getSupplementaryLangs().isEmpty())) {
            return true;
        } else {
            return false;
        }
    }

    public static List<FilterUnit> toFilterUnitList(Corpus corpus, UiComponents uiComponents, boolean addChrono, boolean addIntituleFiche, Predicate<UiComponent> predicate) {
        return initFilterUnitListBuilder(corpus, uiComponents, addChrono, addIntituleFiche, predicate).toFilterUnitList();
    }

    private static FilterUnitListBuilder initFilterUnitListBuilder(Corpus corpus, UiComponents uiComponents, boolean addChrono, boolean addIntituleFiche, Predicate<UiComponent> predicate) {
        Fichotheque fichotheque = corpus.getFichotheque();
        FicheFilter titreFicheFilter = FicheFilterBuilder.TITRE_FICHEFILTER;
        FilterUnitListBuilder builder = new FilterUnitListBuilder();
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (!predicate.test(uiComponent)) {
                continue;
            }
            if (uiComponent instanceof FieldUi) {
                builder.addFieldKeyFilter(((FieldUi) uiComponent).getFieldKey(), null);
            } else if (uiComponent instanceof IncludeUi) {
                IncludeUi includeUi = (IncludeUi) uiComponent;
                ExtendedIncludeKey includeKey = includeUi.getExtendedIncludeKey();
                if (includeKey == null) {
                    switch (includeUi.getName()) {
                        case FichothequeConstants.DATECREATION_NAME:
                        case FichothequeConstants.DATEMODIFICATION_NAME:
                            builder.addChronoFilter();
                            break;
                        case FichothequeConstants.LIAGE_NAME:
                            ExtraitCorpusDef liageExtraitCorpusDef = getLiageExtraitCorpusDef(corpus, UiUtils.checkLiageTest(uiComponents), titreFicheFilter);
                            if (liageExtraitCorpusDef != null) {
                                builder.addExtraitCorpusFilter(liageExtraitCorpusDef);
                            }
                            break;
                        case FichothequeConstants.RATTACHEMENT_NAME:
                            Subset masterSubset = corpus.getMasterSubset();
                            if ((masterSubset != null) && (masterSubset instanceof Thesaurus)) {
                                MotcleFilter motcleFilter = MotcleFilterBuilder.init()
                                        .setWithLabels(true)
                                        .setWithIntitule(ThesaurusUtils.isWithFicheStyle(masterSubset.getMetadata()))
                                        .toMotcleFilter();
                                builder.addMotcleRattachementFilter(getMotcleRattachementDef(motcleFilter));
                            }
                            builder.addFicheRattachementFilter(getFicheRattachementDef(titreFicheFilter));
                            break;
                    }
                } else {
                    switch (includeKey.getCategory()) {
                        case SubsetKey.CATEGORY_CORPUS:
                            builder.addExtraitCorpusFilter(getExtraitCorpusDef(includeKey, titreFicheFilter));
                            break;
                        case SubsetKey.CATEGORY_THESAURUS:
                            boolean withIntitule = false;
                            Subset subset = fichotheque.getSubset(includeKey.getSubsetKey());
                            if (subset != null) {
                                withIntitule = ThesaurusUtils.isWithFicheStyle(subset.getMetadata());
                            }
                            MotcleFilter motcleFilter = MotcleFilterBuilder.init()
                                    .setWithLabels(true)
                                    .setWithIntitule(withIntitule)
                                    .toMotcleFilter();
                            builder.addExtraitThesaurusFilter(getExtraitThesaurusDef(includeKey, motcleFilter));
                            break;
                        case SubsetKey.CATEGORY_ALBUM:
                            IllustrationFilter ficheIllustrationFilter = IllustrationFilterBuilder.init()
                                    .toIllustrationFilter();
                            builder.addExtraitAlbumFilter(getExtraitAlbumDef(includeKey, ficheIllustrationFilter));
                            break;
                        case SubsetKey.CATEGORY_ADDENDA:
                            DocumentFilter ficheDocumentFilter = DocumentFilterBuilder.init()
                                    .toDocumentFilter();
                            builder.addExtraitAddendaFilter(getExtraitAddendaDef(includeKey, ficheDocumentFilter));
                            break;

                    }
                }
            }
        }
        if (addChrono) {
            builder.addChronoFilter();
        }
        if (addIntituleFiche) {
            builder.addIntituleFilter();
        }
        return builder;
    }


    private static ExtractionDef getCompilationExtractionDef(BdfServer bdfServer) {
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        Fichotheque fichotheque = bdfServer.getFichotheque();
        UiManager uiManager = bdfServer.getUiManager();
        for (Corpus corpus : fichotheque.getCorpusList()) {
            UiComponents uiComponents = uiManager.getMainUiComponents(corpus);
            List<FilterUnit> filterUnitList = toFilterUnitList(corpus, uiComponents, true, true, UiUtils.ALL_COMPONENTPREDICATE);
            FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(filterUnitList);
            FicheQuery ficheQuery = FicheQueryBuilder.init()
                    .addCorpus(corpus)
                    .toFicheQuery();
            ExtraitCorpusDef extraitCorpusDef = ExtraitCorpusDefBuilder.init(ficheFilter)
                    .setIntituleClause(IntituleClauseBuilder.DEFAULT)
                    .setFicheQueryList(ficheQuery)
                    .toExtraitCorpusDef();
            extractionDefBuilder.addStatic(extraitCorpusDef);
        }
        return extractionDefBuilder.toExtractionDef();
    }

    private static ExtractionDef getFicheExtractionDef(BdfServer bdfServer, SubsetKey corpusKey) {
        Corpus corpus = (Corpus) bdfServer.getFichotheque().getSubset(corpusKey);
        UiComponents uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        List<Corpus> includedCorpusList = BdfServerUtils.getIncludeSatelliteList(corpus);
        Predicate<UiComponent> predicate;
        if (!includedCorpusList.isEmpty()) {
            predicate = UiUtils.MAIN_COMPONENTPREDICATE;
        } else {
            predicate = UiUtils.ALL_COMPONENTPREDICATE;
        }
        FilterUnitListBuilder builder = initFilterUnitListBuilder(corpus, uiComponents, true, true, predicate);
        for (Corpus satellite : includedCorpusList) {
            List<FilterUnit> satelliteFilterUnitList = toFilterUnitList(satellite, bdfServer.getUiManager().getMainUiComponents(satellite), false, false, UiUtils.SATELLITE_COMPONENTPREDICATE);
            FicheFilter satelliteFicheFilter = FicheFilterBuilder.fromFilterUnitList(satelliteFilterUnitList);
            FicheRattachementDef ficheRattachementDef = FicheRattachementDefBuilder.init(satelliteFicheFilter).addRattachementSubsetKey(satellite.getSubsetKey()).toFicheRattachementDef();
            builder.addFicheRattachementFilter(ficheRattachementDef);
        }
        FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(builder.toFilterUnitList());
        return ExtractionDefBuilder.init()
                .setDynamicExtraitCorpusDef(BuilderUtils.getExtraitCorpusDef(ficheFilter, null))
                .toExtractionDef();
    }

    static Set<String> getDefaultStreamTemplateExtensionSet(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey()) {
            return DEFAULT_SET;
        }
        if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            return DEFAULT_SET;
        }
        return EMPTY_SET;
    }

    private static ExtraitCorpusDef getExtraitCorpusDef(ExtendedIncludeKey includeKey, FicheFilter ficheFilter) {
        FicheQuery ficheQuery = FicheQueryBuilder.init()
                .addCorpus(includeKey.getSubsetKey())
                .setCroisementCondition(SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter()))
                .toFicheQuery();
        return ExtraitCorpusDefBuilder.init(ficheFilter)
                .setExtraitName(includeKey.toString())
                .setMaster(includeKey.isMaster())
                .setFicheQueryList(ficheQuery)
                .toExtraitCorpusDef();
    }

    private static ExtraitCorpusDef getLiageExtraitCorpusDef(Corpus referenceCorpus, LiageTest liageTest, FicheFilter ficheFilter) {
        List<SubsetKey> allCorpusList = new ArrayList<SubsetKey>();
        List<FicheQuery> ficheQueryList = new ArrayList<FicheQuery>();
        for (Corpus otherCorpus : referenceCorpus.getFichotheque().getCorpusList()) {
            SubsetKey otherKey = otherCorpus.getSubsetKey();
            Object testResult = liageTest.ownsToLiage(otherKey);
            if (testResult instanceof Ranges) {
                Ranges excludeRanges = (Ranges) testResult;
                FicheQuery ficheQuery = FicheQueryBuilder.init()
                        .addCorpus(otherKey)
                        .setCroisementCondition(SelectionUtils.toCroisementCondition("", excludeRanges, true))
                        .toFicheQuery();
                ficheQueryList.add(ficheQuery);
            } else {
                Boolean bool = (Boolean) testResult;
                if (bool) {
                    allCorpusList.add(otherKey);
                }
            }
        }
        if (!allCorpusList.isEmpty()) {
            FicheQuery ficheQuery = FicheQueryBuilder.init()
                    .addCorpus(allCorpusList)
                    .setCroisementCondition(SelectionUtils.toCroisementCondition("", null, true))
                    .toFicheQuery();
            ficheQueryList.add(ficheQuery);
        }
        if (ficheQueryList.isEmpty()) {
            return null;
        }
        return ExtraitCorpusDefBuilder.init(ficheFilter)
                .setExtraitName(FichothequeConstants.LIAGE_NAME)
                .setFicheQueryList(ficheQueryList)
                .toExtraitCorpusDef();
    }

    private static FicheRattachementDef getFicheRattachementDef(FicheFilter ficheFilter) {
        return FicheRattachementDefBuilder.init(ficheFilter)
                .toFicheRattachementDef();
    }

    private static ExtraitThesaurusDef getExtraitThesaurusDef(ExtendedIncludeKey includeKey, MotcleFilter motcleFilter) {
        MotcleQuery motcleQuery = MotcleQueryBuilder.init()
                .addThesaurus(includeKey.getSubsetKey())
                .setCroisementCondition(SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter()))
                .setIncludeSatellites(false)
                .toMotcleQuery();
        return ExtraitThesaurusDefBuilder.init(motcleFilter)
                .setExtraitName(includeKey.toString())
                .setMaster(includeKey.isMaster())
                .setMotcleQueryList(motcleQuery)
                .toExtraitThesaurusDef();
    }

    private static MotcleRattachementDef getMotcleRattachementDef(MotcleFilter motcleFilter) {
        return MotcleRattachementDefBuilder.init(motcleFilter)
                .toMotcleRattachementDef();
    }

    private static ExtraitAlbumDef getExtraitAlbumDef(ExtendedIncludeKey includeKey, IllustrationFilter illustrationFilter) {
        IllustrationQuery illustrationQuery = IllustrationQueryBuilder.init()
                .addAlbum(includeKey.getSubsetKey())
                .setCroisementCondition(SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter()))
                .toIllustrationQuery();
        return ExtraitAlbumDefBuilder.init(illustrationFilter)
                .setExtraitName(includeKey.toString())
                .setIllustrationQueryList(illustrationQuery)
                .toExtraitAlbumDef();
    }

    private static ExtraitAddendaDef getExtraitAddendaDef(ExtendedIncludeKey includeKey, DocumentFilter documentFilter) {
        DocumentQuery documentQuery = DocumentQueryBuilder.init()
                .addAddenda(includeKey.getSubsetKey())
                .setCroisementCondition(SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter()))
                .toDocumentQuery();
        return ExtraitAddendaDefBuilder.init(documentFilter)
                .setExtraitName(includeKey.toString())
                .setDocumentQueryList(documentQuery)
                .toExtraitAddendaDef();
    }

    private static String getShouldNotOccurMessage(TemplateCompiler templateCompiler) {
        List<TemplateContentDescription> list = templateCompiler.flushTemplateContentDescriptionList();
        StringBuilder buf = new StringBuilder();
        for (TemplateContentDescription description : list) {
            for (MessageByLine messageByLine : description.getMessageByLineList()) {
                for (MessageByLine.Category category : messageByLine.getCategoryList()) {
                    if (buf.length() > 0) {
                        buf.append(" @@@ ");
                    }
                    buf.append(category.getName());
                    for (Message message : category.getMessageList()) {
                        buf.append(" | ");
                        buf.append(message.getMessageKey());
                        Object[] values = message.getMessageValues();
                        for (Object value : values) {
                            buf.append(" / ");
                            buf.append(value);
                        }
                    }
                }
            }
        }
        if (buf.length() == 0) {
            buf.append("Unknown compilation error");
        }
        return buf.toString();
    }


    private static class ShouldNotOccurMessageHandler implements MessageHandler {

        private ShouldNotOccurMessageHandler() {
        }

        @Override
        public void addMessage(String category, Message message) {
            StringBuilder buf = new StringBuilder();
            buf.append(category);
            buf.append(" / ");
            buf.append(message.getMessageKey());
            Object[] values = message.getMessageValues();
            for (Object value : values) {
                buf.append(" / ");
                buf.append(value);
            }
            throw new ShouldNotOccurException(buf.toString());
        }

    }


    private static class InternalURIResolver implements URIResolver {

        private final URIResolver originalUriResolver;

        private InternalURIResolver(URIResolver originalUriResolver) {
            this.originalUriResolver = originalUriResolver;
        }

        @Override
        public Source resolve(String href, String base) throws TransformerException {
            Source source = originalUriResolver.resolve(href, base);
            if (source == null) {
                throw new TransformerException("(base: " + base + ")");
            } else {
                return source;
            }
        }

    }

    private static String getAction(String extension) {
        switch (extension) {
            case "ods":
                return "action-Ods";
            case "odt":
                return "action-Odt";
            default:
                return "action-Test";
        }
    }

}
