/* BdfServer - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.storage.StorageContent;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.transformation.TemplateDef;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.tools.exportation.transformation.TemplateDefBuilder;
import net.fichotheque.xml.ExtractionDefXMLPart;
import net.mapeadores.util.xml.XsltWriter;


/**
 *
 * @author Vincent Calame
 */
class XsltHtmlUnit {

    private XsltHtmlUnit() {
    }

    static TemplateStorage.Unit build(BdfServer bdfServer, TemplateKey templateKey, boolean withIf, boolean withExtractionDef) {
        if (!templateKey.isSimpleTemplate()) {
            throw new IllegalArgumentException("templateKey is not a simple template key");
        }
        TransformationKey transformationKey = templateKey.getTransformationKey();
        TemplateDef templateDef = getTemplateDef(templateKey);
        List<StorageContent> list = new ArrayList<StorageContent>();
        String xslt;
        if (isFicheXslt(transformationKey)) {
            xslt = getFicheXslt(bdfServer, transformationKey, withIf);
        } else {
            xslt = getTransformationXslt(transformationKey);
        }
        list.add(StorageUtils.toStorageContent("transformer.xsl", xslt));
        if (withExtractionDef) {
            ExtractionDef extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, transformationKey);
            if (extractionDef != null) {
                String extractionContent = ExtractionDefXMLPart.toString(extractionDef, true, false);
                list.add(StorageUtils.toStorageContent("extraction.xml", extractionContent));
            }
        }
        List<StorageContent> finalList = StorageUtils.wrap(list.toArray(new StorageContent[list.size()]));
        return new DefaultTemplateStorageUnit("xslt", templateKey, templateDef, finalList);
    }

    private static TemplateDef getTemplateDef(TemplateKey templateKey) {
        return TemplateDefBuilder.init(templateKey).toTemplateDef();
    }

    private static boolean isFicheXslt(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey()) {
            return true;
        }
        if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            return true;
        }
        return false;
    }

    private static String getTransformationXslt(TransformationKey transformationKey) {
        StringBuilder buf = new StringBuilder();
        XsltWriter xsltWriter = new XsltWriter();
        xsltWriter.setAppendable(buf);
        xsltWriter.setIndentLength(0);
        try {
            xsltWriter.appendXMLDeclaration();
            xsltWriter.openXslStyleSheet();
            xsltWriter.addXslInclude("bdf://this/xslt/" + transformationKey.toString() + ".xsl");
            xsltWriter.closeXslStyleSheet();
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    private static String getFicheXslt(BdfServer bdfServer, TransformationKey transformationKey, boolean withIf) {
        StringBuilder buf = new StringBuilder();
        SimpleFicheXsltWriter ficheXsltWriter = new SimpleFicheXsltWriter();
        ficheXsltWriter.setAppendable(buf);
        try {
            ficheXsltWriter.appendXMLDeclaration();
            ficheXsltWriter.openXslStyleSheet();
            if (transformationKey.isCorpusTransformationKey()) {
                ficheXsltWriter.addXslInclude("bdf://this/xslt/fiche.xsl");
                SubsetKey corpusKey = transformationKey.toCorpusKey();
                Corpus corpus = (Corpus) bdfServer.getFichotheque().getSubset(corpusKey);
                ficheXsltWriter.add(bdfServer, corpus, false, withIf);
            } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
                ficheXsltWriter.addXslInclude("bdf://this/xslt/compilation.xsl");
                for (Corpus corpus : bdfServer.getFichotheque().getCorpusList()) {
                    ficheXsltWriter.add(bdfServer, corpus, true, withIf);
                }
            }
            ficheXsltWriter.closeXslStyleSheet();
        } catch (IOException ioe) {
        }
        return buf.toString();
    }


}
