/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform.builders;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.HiddenIncludeElement;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.tools.ficheform.FicheFormUtils;
import java.util.Collection;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class HiddenIncludeElementBuilder {

    private final String name;
    private String label = "";
    private boolean mandatory = false;
    private String value = "";
    private Attributes attributes = AttributeUtils.EMPTY_ATTRIBUTES;

    public HiddenIncludeElementBuilder(String name) {
        this.name = name;
    }

    public HiddenIncludeElementBuilder setLabel(String label) {
        if (label == null) {
            throw new IllegalArgumentException("label is null");
        }
        this.label = label;
        return this;
    }

    public HiddenIncludeElementBuilder setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
        return this;
    }

    public HiddenIncludeElementBuilder setAttributes(Attributes attributes) {
        if (attributes == null) {
            this.attributes = AttributeUtils.EMPTY_ATTRIBUTES;
        } else {
            this.attributes = attributes;
        }
        return this;
    }

    public HiddenIncludeElementBuilder setValue(String value) {
        if (value == null) {
            throw new IllegalArgumentException("value is null");
        }
        this.value = value;
        return this;
    }

    public HiddenIncludeElement toHiddenIncludeElement() {
        return new InternalHiddenIncludeElement(name, label, mandatory, attributes, value);
    }

    @Nullable
    public static HiddenIncludeElementBuilder check(FichePointeur fichePointeur, FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        ExtendedIncludeKey extendedIncludeKey = includeUi.getExtendedIncludeKey();
        if (extendedIncludeKey == null) {
            return null;
        }
        Thesaurus thesaurus = (Thesaurus) ficheFormParameters.getBdfServer().getFichotheque().getSubset(extendedIncludeKey.getSubsetKey());
        if (thesaurus == null) {
            return null;
        }
        if (!FicheFormUtils.hasAccess(ficheFormParameters, includeUi)) {
            return null;
        }
        SubsetItemPointeur pointeur = FicheFormUtils.checkMasterPointeur(fichePointeur, ficheFormParameters, extendedIncludeKey);
        Collection<Liaison> liaisonList = FicheFormUtils.initMotcleList(pointeur, thesaurus, extendedIncludeKey.getRootIncludeKey());
        String value;
        if (pointeur.isEmpty()) {
            value = FicheFormUtils.getDefVal(ficheFormParameters, includeUi);
        } else {
            value = FicheFormUtils.getStringValue(thesaurus, liaisonList, extendedIncludeKey.hasPoidsFilter(), ficheFormParameters);
        }
        return init(includeUi.getName())
                .setValue(value)
                .setAttributes(includeUi.getAttributes());
    }

    public static HiddenIncludeElementBuilder init(String name) {
        return new HiddenIncludeElementBuilder(name);
    }


    private static class InternalHiddenIncludeElement implements HiddenIncludeElement {

        private final String name;
        private final String label;
        private final boolean mandatory;
        private final Attributes attributes;
        private final String value;

        private InternalHiddenIncludeElement(String name, String label, boolean mandatory, Attributes attributes, String value) {
            this.name = name;
            this.label = label;
            this.mandatory = mandatory;
            this.attributes = attributes;
            this.value = value;
        }

        @Override
        public String getIncludeName() {
            return name;
        }

        @Override
        public String getLabel() {
            return label;
        }

        @Override
        public boolean isMandatory() {
            return mandatory;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public String getValue() {
            return value;
        }

    }

}
