/* BdfServer_Html - Copyright (c) 2020-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.consumers;

import fr.exemole.bdfserver.api.BdfExtensionReference;
import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.providers.MetadataPhraseDefProvider;
import fr.exemole.bdfserver.api.ui.MetadataPhraseDef;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.ui.MetadataPhraseDefBuilder;
import fr.exemole.bdfserver.tools.ui.MetadataPhraseDefCatalog;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.function.Consumer;
import net.fichotheque.Metadata;
import net.fichotheque.namespaces.BdfSpace;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.localisation.Litteral;
import net.mapeadores.util.text.Labels;
import net.mapeadores.util.text.Phrase;


/**
 *
 * @author Vincent Calame
 */
public class MetadataPhrases implements Consumer<HtmlPrinter> {

    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String NEWPHRASE_PARAMNAME = "newphrase";
    public final static String NEWPHRASE_PARAMPREFIX = "newphrase/";
    private final Metadata metadata;
    private final Langs langs;
    private final Object titleL10nObject;
    private final Map<String, MetadataPhraseDef> parametersMap = new LinkedHashMap<String, MetadataPhraseDef>();
    private boolean withNewPhrase = false;
    private String dlCssClasses = "global-DL";

    public MetadataPhrases(Metadata metadata, Langs langs, Object titleL10nObject) {
        this.metadata = metadata;
        this.langs = langs;
        this.titleL10nObject = titleL10nObject;
    }

    public boolean containsPhrase(String phraseName) {
        return parametersMap.containsKey(phraseName);
    }

    public MetadataPhrases addDef(MetadataPhraseDef metadataPhraseDef) {
        parametersMap.put(metadataPhraseDef.getName(), metadataPhraseDef);
        return this;
    }

    public MetadataPhrases withNewPhrase() {
        withNewPhrase = true;
        return this;
    }

    public MetadataPhrases dlCssClasses(String dlCssClasses) {
        this.dlCssClasses = dlCssClasses;
        return this;
    }

    public MetadataPhrases addExtensionPhraseDefList(BdfServer bdfServer) {
        for (BdfExtensionReference extensionReference : bdfServer.getExtensionManager().getBdfExtensionReferenceList()) {
            MetadataPhraseDefProvider provider = (MetadataPhraseDefProvider) extensionReference.getImplementation(MetadataPhraseDefProvider.class);
            if (provider != null) {
                for (MetadataPhraseDef phraseDef : provider.getMetadataPhraseDefList(metadata)) {
                    addDef(phraseDef);
                }
            }
        }
        return this;
    }

    @Override
    public void accept(HtmlPrinter hp) {
        Set<String> done = new HashSet<String>();
        hp
                .DL(dlCssClasses);
        addPhrase(hp, InteractionConstants.TITLE_PARAMPREFIX, metadata.getTitleLabels(), titleL10nObject, 30);
        for (MetadataPhraseDef def : parametersMap.values()) {
            String name = def.getName();
            done.add(name);
            addPhrase(hp, def);
        }
        Attribute attribute = metadata.getAttributes().getAttribute(BdfSpace.PHRASES_KEY);
        if (attribute != null) {
            for (String value : attribute) {
                MetadataPhraseDef def = toMetadataPhraseDef(value);
                if (def != null) {
                    String name = def.getName();
                    if (!done.contains(name)) {
                        done.add(name);
                        addPhrase(hp, def);
                    }
                }
            }
        }
        for (Phrase phrase : metadata.getPhrases()) {
            String name = phrase.getName();
            if (!done.contains(name)) {
                done.add(name);
                addPhrase(hp, BdfInstructionUtils.getPhraseParamPrefix(name), phrase, new Litteral("[" + name + "]"), 30);
            }
        }
        if (withNewPhrase) {
            HtmlAttributes inputAttributes = hp.name(InteractionConstants.NEWPHRASE_PARAMNAME).size("15");
            hp
                    .DT()
                    .SPAN("command-FlexInput")
                    .LABEL_for(inputAttributes.id())
                    .__localize("_ label.global.newphrase")
                    ._LABEL()
                    .SPAN("")
                    .__colon()
                    ._SPAN()
                    .INPUT_text(inputAttributes)
                    ._SPAN()
                    ._DT()
                    .DD()
                    .__(Grid.START)
                    .__(LangRows.init(InteractionConstants.NEWPHRASE_PARAMPREFIX, null, langs).cols(30))
                    .__(Grid.END)
                    ._DD();
        }
        hp
                ._DL();
    }

    private void addPhrase(HtmlPrinter hp, MetadataPhraseDef def) {
        String name = def.getName();
        addPhrase(hp, BdfInstructionUtils.getPhraseParamPrefix(name), metadata.getPhrases().getPhrase(name), def.getL10nObject(), def.getSize());
    }

    private void addPhrase(HtmlPrinter hp, String paramPrefix, Labels labels, Object l10nObject, int size) {
        if ((l10nObject instanceof Litteral) && (l10nObject.toString().startsWith("["))) {
            hp
                    .DT()
                    .CODE()
                    .__localize(l10nObject)
                    ._CODE()
                    ._DT();
        } else {
            hp
                    .DT()
                    .__localize(l10nObject)
                    ._DT();
        }
        hp
                .DD()
                .__(Grid.START)
                .__(LangRows.init(paramPrefix, labels, langs).cols(size))
                .__(Grid.END)
                ._DD();
    }

    public static MetadataPhrases init(Metadata metadata, Langs langs, String titleLocKey) {
        return new MetadataPhrases(metadata, langs, titleLocKey);
    }

    private MetadataPhraseDef toMetadataPhraseDef(String value) {
        String name = value;
        int idx = value.indexOf('|');
        if (idx == -1) {
            MetadataPhraseDef def = MetadataPhraseDefCatalog.getMetadataPhraseDef(name);
            if (def != null) {
                return def;
            } else {
                if (!FichothequeUtils.isValidPhraseName(name)) {
                    return null;
                }
                return MetadataPhraseDefBuilder.init(name).toMetadataPhraseDef();
            }
        } else {
            name = value.substring(0, idx).trim();
            if (!FichothequeUtils.isValidPhraseName(name)) {
                return null;
            }
            String l10nKey = value.substring(idx + 1);
            int size = 30;
            int idx2 = l10nKey.indexOf('|');
            if (idx2 > 0) {
                try {
                    size = Integer.parseInt(l10nKey.substring(idx2 + 1).trim());
                } catch (NumberFormatException nfe) {

                }
                if (size < 1) {
                    size = 30;
                }
                l10nKey = l10nKey.substring(0, idx2);
            }
            l10nKey = l10nKey.trim();
            if (l10nKey.isEmpty()) {
                l10nKey = null;
            }
            return MetadataPhraseDefBuilder.init(name).setL10nObject(l10nKey).setSize(size).toMetadataPhraseDef();
        }
    }

}
