/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.ExportationDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.balayage.engine.BalayageLog;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class ExportationHtmlProducerFactory {

    private ExportationHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (page) {
            case ExportationDomain.BALAYAGE_INDEX_PAGE: {
                checker.checkGlobalAdmin();
                return new BalayageIndexHtmlProducer(checker);
            }
            case ExportationDomain.BALAYAGE_ERROR_PAGE: {
                Balayage balayage = getBalayage(bdfServer, requestMap, bdfCommandResult);
                return new BalayageErrorHtmlProducer(checker, balayage);
            }
            case ExportationDomain.BALAYAGE_RESULT_PAGE: {
                Balayage balayage = getBalayage(bdfServer, requestMap, bdfCommandResult);
                BalayageLog balayageLog = getBalayageLog(bdfCommandResult);
                String mode = getMode(bdfCommandResult);
                return new BalayageResultHtmlProducer(checker, balayage, mode, balayageLog);
            }
            case ExportationDomain.TABLEEXPORTS_PAGE: {
                checker.checkGlobalAdmin();
                return new TableExportsHtmlProducer(checker);
            }
            case ExportationDomain.SCRUTARIEXPORTS_PAGE: {
                checker.checkGlobalAdmin();
                return new ScrutariExportsHtmlProducer(checker);
            }
            case ExportationDomain.TRANSFORMATIONS_PAGE: {
                checker.checkGlobalAdmin();
                return new TransformationsHtmlProducer(checker);
            }
            case ExportationDomain.TRANSFORMATION_BINARYUPDATE_PAGE: {
                checker.checkGlobalAdmin();
                TemplateContentDescription templateContentDescription = getTemplateContentDescription(bdfServer, requestMap, bdfCommandResult);
                return new TemplateBinaryUpdateHtmlProducer(checker, templateContentDescription);
            }
            case ExportationDomain.SQLEXPORTS_PAGE: {
                checker.checkGlobalAdmin();
                return new SqlExportsHtmlProducer(checker);
            }
            case ExportationDomain.BALAYAGES_PAGE: {
                checker.checkGlobalAdmin();
                return new BalayagesHtmlProducer(checker);
            }
            default:
                return null;
        }
    }

    private static Balayage getBalayage(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Balayage balayage = null;
        if (bdfCommandResult != null) {
            balayage = (Balayage) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGE_OBJ);
        }
        if (balayage == null) {
            balayage = BdfInstructionUtils.getMandatoryBalayage(bdfServer.getBalayageManager(), requestMap);
        }
        return balayage;
    }

    private static BalayageLog getBalayageLog(BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            return (BalayageLog) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGELOG_OBJ);
        }
        return null;
    }

    private static String getMode(BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            return (String) bdfCommandResult.getResultObject(BdfInstructionConstants.MODE_OBJ);
        }
        return null;
    }

    private static TemplateContentDescription getTemplateContentDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        TemplateContentDescription templateContentDescription = null;
        if (bdfCommandResult != null) {
            templateContentDescription = (TemplateContentDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TEMPLATECONTENTDESCRIPTION_OBJ);
        }
        if (templateContentDescription == null) {
            TemplateDescription templateDescription = null;
            if (bdfCommandResult != null) {
                templateDescription = (TemplateDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
            }
            if (templateDescription == null) {
                templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(bdfServer, requestMap);
            }
            templateContentDescription = BdfInstructionUtils.getMandatoryTemplateContentDescription(templateDescription, requestMap);
        }
        return templateContentDescription;
    }

}
