/* BdfServer_HtmlProducers - Copyright (c) 2019-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.main;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.AdministrationDomain;
import fr.exemole.bdfserver.api.interaction.domains.MainDomain;
import fr.exemole.bdfserver.api.interaction.domains.MiscDomain;
import fr.exemole.bdfserver.api.interaction.domains.SelectionDomain;
import fr.exemole.bdfserver.api.interaction.domains.SphereDomain;
import fr.exemole.bdfserver.api.menu.MenuGroup;
import fr.exemole.bdfserver.api.menu.MenuLink;
import fr.exemole.bdfserver.api.menu.MenuNode;
import fr.exemole.bdfserver.api.providers.MenuLinkProvider;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.subsettree.SubsetTree;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlConstants;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.Button;
import fr.exemole.bdfserver.html.consumers.SubsetTreeOptions;
import fr.exemole.bdfserver.html.jslib.MiscJsLibs;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.BH;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import fr.exemole.bdfserver.tools.roles.RoleUtils;
import fr.exemole.bdfserver.tools.subsettree.TreeFilterEngine;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.FichothequeMetadataUtils;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.localisation.Message;


/**
 *
 * @author Vincent Calame
 */
public class IframesHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Button linkButton = Button.link();

    public IframesHtmlProducer(ProducerParameters producerParameters) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser());
    }

    public IframesHtmlProducer(BdfServer bdfServer, BdfUser bdfUser) {
        super(bdfServer, bdfUser);
        addThemeCss("main.css");
        addJsLib(MiscJsLibs.MAIN);
        addJsLib(MiscJsLibs.MENU);
        this.permissionSummary = PermissionSummaryBuilder.build(bdfServer, bdfUser);
        setMainStorageKey(Domains.MAIN, MainDomain.IFRAMES_PAGE);
    }

    @Override
    public void printHtml() {
        start(FichothequeMetadataUtils.getLongTitle(bdfServer.getFichotheque(), workingLang));
        this
                .DIV(HA.id("main"))
                .DIV(HA.id("main_menu").classes("menu-MenuBar"))
                .__(printLogoBlock())
                .__(printListBlock())
                .__(printAdminBlock())
                .__(printActionBlock())
                .__(printFicheBlock())
                .__(printGlue())
                .__(printSessionBlock())
                ._DIV()
                .DIV(HA.id("main_body"))
                .DIV(HA.id("main_body_tool"))
                ._DIV()
                .DIV(HA.id("main_body_list"))
                .IFRAME(HA.name(BdfHtmlConstants.LIST_FRAME).id("main_body_list_iframe").src(BH.domain(Domains.MAIN).page(MainDomain.FICHES_PAGE)))
                ._IFRAME()
                ._DIV()
                .DIV(HA.id("main_body_edition"))
                .IFRAME(HA.name(BdfHtmlConstants.EDITION_FRAME).id("main_body_edition_iframe").src(BH.domain(Domains.MAIN).page(MainDomain.START_PAGE)))
                ._IFRAME()
                ._DIV()
                ._DIV()
                ._DIV();
        end();
    }

    private boolean printLogoBlock() {
        String sessionString = getSessionString();
        this
                .IMG(HA.src("images/icon32.png").classes("menu-Logo").title(sessionString).alt(sessionString).width("32").height("32"));
        return true;
    }

    private boolean printListBlock() {
        String ficheHref = BH.domain(Domains.MAIN).page(MainDomain.FICHES_PAGE).param(MainDomain.FICHES_RELOAD_PARAMNAME, "1").toString();
        String collectionsHref = BH.domain(Domains.MAIN).page(MainDomain.COLLECTIONS_PAGE).toString();
        this
                .DIV("menu-ListBlock")
                .__(listLink(ficheHref, "action-Fiches", "_ link.main.fiches"))
                .__(listLink(collectionsHref, "action-Collections", "_ link.main.collections"))
                ._DIV();
        return true;
    }

    private boolean printAdminBlock() {
        String selectionHref = BH.domain(Domains.SELECTION).page(SelectionDomain.SELECTIONFORM_PAGE).toString();
        String adminHref = BH.domain(Domains.ADMINISTRATION).page(AdministrationDomain.INDEX_PAGE).toString();
        this
                .DIV("menu-AdminBlock")
                .__(editionLink(selectionHref, "action-Selection", "_ link.selection.form"));
        if ((permissionSummary.isGlobalAdmin()) || (RoleUtils.isPartialAdmin(permissionSummary))) {
            this
                    .__(listLink(adminHref, "action-Admin", "_ link.administration.index"));
        } else {
            this
                    .SPAN("menu-EmptySpace")
                    ._SPAN();
        }
        this
                ._DIV();
        return true;
    }

    private boolean printActionBlock() {
        this
                .DIV("menu-ActionBlock")
                .__(printActionSelect())
                .SPAN(HA.id("redoArea").classes("menu-EmptySpace"))
                ._SPAN()
                ._DIV();
        return true;
    }

    private boolean printFicheBlock() {
        SubsetTree filteredSubsetTree = TreeFilterEngine.read(permissionSummary, bdfServer.getTreeManager().getSubsetTree(SubsetKey.CATEGORY_CORPUS));
        this
                .DIV(HA.attr("data-role", "ficheBlock").classes("menu-FicheBlock").addClass(filteredSubsetTree.getNodeList().isEmpty(), "invisible"));
        this
                .DIV("menu-fiche-CreationForm");
        this
                .FORM_get(Domains.EDITION, BdfHtmlConstants.EDITION_FRAME)
                .SELECT(HA.id("corpusSelect").name("corpus"))
                .__(SubsetTreeOptions.init(filteredSubsetTree, bdfServer, workingLang)
                        .onlyNames(true)
                        .withKeys(false)
                )
                ._SELECT()
                .__space()
                .__(printGotoButton());
        this
                ._FORM();
        this
                ._DIV();
        this
                .DIV("menu-fiche-DirectAccess")
                .__localize("_ label.menu.goto")
                .__colon()
                .INPUT_text(HA.id("ficheGotoInput").name("goto").size("6"))
                .__(printAccessButton("read", "action-FicheDisplay", "_ link.fiches.fiche_long", BdfHtmlConstants.EDITION_FRAME))
                .__(printAccessButton("write", "action-FicheEdit", "_ link.edition.fichechange_long", BdfHtmlConstants.EDITION_FRAME))
                .__((permissionSummary.isGlobalAdmin()) ? printAccessButton("indexation", "action-FicheIndexation", "_ link.edition.ficheindexation_long", BdfHtmlConstants.EDITION_FRAME) : false)
                .__(printAccessButton("selection", "action-FicheSelection", "_ link.selection.byfiche", BdfHtmlConstants.LIST_FRAME))
                ._DIV();
        this
                ._DIV();
        return true;
    }

    private boolean printGlue() {
        this
                .DIV("global-Glue")
                ._DIV();
        return true;
    }

    private boolean printSessionBlock() {
        this
                .DIV("menu-SessionBlock")
                .__(printSessionButton(HA.href(BH.domain(Domains.SESSION).param(InteractionConstants.BDF_EXIT_PARAMETER, "1").param(InteractionConstants.BDF_SPHERE_PARAMETER, bdfUser.getRedacteur().getSphere().getSubsetKey().getSubsetName())).target(HtmlConstants.TOP_TARGET), "action-SessionClose", "_ link.session.new"))
                .__(printSessionButton(HA.href(BH.domain(Domains.SESSION)).target(HtmlConstants.BLANK_TARGET), "action-SessionDuplicate", "_ link.session.duplicate"))
                ._DIV();
        return true;
    }

    private boolean printActionSelect() {
        this
                .SELECT(HA.id("actionSelect").classes("menu-action-Select"));
        this
                .OPTION(HA.value(""))
                .__localize("_ title.menu.actions")
                ._OPTION()
                .__(printActionOption(BH.domain(Domains.MISC).page(MiscDomain.TABLEEXPORTFORM_PAGE).toString(), "action-Ods", "_ link.misc.ods_export", BdfHtmlConstants.EDITION_FRAME))
                .__(printActionOption(BH.domain(Domains.SPHERE).page(SphereDomain.REDACTEUR_USERFORM_PAGE).toString(), "action-Preferences", "_ link.sphere.userprefs", BdfHtmlConstants.EDITION_FRAME))
                .__(printActionOption(BH.domain(Domains.MISC).page(MiscDomain.MEMENTO_PAGE).toString(), "action-Memento", "_ link.misc.memento", BdfHtmlConstants.LIST_FRAME));
        List<MenuGroup> menuGroupList = BdfServerUtils.getExtensionMenuGroupList(bdfServer, bdfUser, MenuLinkProvider.MAINMENU_KEY);
        for (MenuGroup menuGroup : menuGroupList) {
            this
                    .__(printMenuGroup(menuGroup));
        }
        this
                ._SELECT();
        return true;
    }

    private Button listLink(String href, String action, String messageKey) {
        return linkButton.href(href).action(action).titleMessage(messageKey).target(BdfHtmlConstants.LIST_FRAME);
    }

    private Button editionLink(String href, String action, String messageKey) {
        return linkButton.href(href).action(action).titleMessage(messageKey).target(BdfHtmlConstants.EDITION_FRAME);
    }

    private boolean printMenuGroup(MenuGroup menuGroup) {
        String groupLabel;
        Message titleMessage = menuGroup.getTitleMessage();
        if (titleMessage != null) {
            groupLabel = getLocalization(titleMessage);
        } else {
            groupLabel = menuGroup.getTitle();
        }
        this
                .OPTGROUP(HA.label(groupLabel));
        for (MenuNode menuNode : menuGroup.getMenuNodeList()) {
            if (menuNode instanceof MenuLink) {
                this
                        .__(printOption((MenuLink) menuNode));
            }
        }
        this
                ._OPTGROUP();
        return true;
    }

    private boolean printActionOption(String value, String action, String messageKey, String target) {
        this
                .OPTION(HA.value(value).attr("data-action", action).attr("data-target", target))
                .__localize(messageKey)
                ._OPTION();
        return true;
    }

    private boolean printOption(MenuLink menuLink) {
        String target = menuLink.getTarget();
        if (target == null) {
            target = BdfHtmlConstants.EDITION_FRAME;
        }
        String url = menuLink.getUrl();
        HtmlAttributes optionAttr = HA.value(url).attr("data-target", target);
        String iconPath = menuLink.getIconPath();
        if (iconPath != null) {
            optionAttr.attr("data-icon-src", iconPath);
        }
        this
                .OPTION(optionAttr)
                .__(BdfHtmlUtils.printTitle(this, menuLink))
                ._OPTION();
        return true;
    }

    private boolean printSessionButton(HtmlAttributes linkAttr, String action, String titleLocKey) {
        linkAttr.classes("global-button-Transparent " + action).titleLocKey(titleLocKey);
        this
                .A(linkAttr)
                .__(Button.ICON)
                ._A();
        return true;
    }

    private boolean printAccessButton(String key, String action, String titleLocKey, String target) {
        HtmlAttributes attr = HA.href("#").classes("global-button-Transparent " + action).titleLocKey(titleLocKey).target(target).attr("data-fiche-goto-key", key);
        this
                .A(attr)
                .__(Button.ICON)
                ._A();
        return true;
    }

    private String getSessionString() {
        Redacteur redacteur = bdfUser.getRedacteur();
        StringBuilder buf = new StringBuilder();
        buf.append(getLocalization("_ label.menu.openby"));
        buf.append(" ");
        buf.append(SphereUtils.toCompleteName(redacteur));
        buf.append(" – ");
        buf.append(SphereUtils.toBracketStyle(redacteur));
        return buf.toString();
    }

    private boolean printGotoButton() {
        HtmlAttributes aAttributes = HA.href("#").target(BdfHtmlConstants.EDITION_FRAME).attr("data-fiche-goto-key", "create").classes("global-button-Link action-FicheCreate");
        this
                .A(aAttributes)
                .__(Button.ICON)
                .SPAN("global-button-Text")
                .__localize("_ link.edition.fichecreation")
                ._SPAN()
                ._A();
        return true;
    }

}
