/* FichothequeLib_Tools - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.dom;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.fichotheque.SubsetItem;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.CroisementChange;
import net.fichotheque.croisement.CroisementEditor;
import net.fichotheque.croisement.Lien;
import net.fichotheque.utils.CroisementUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class CroisementDOMReader {

    private final CroisementEditor croisementEditor;
    private final MessageHandler messageHandler;
    private final AttributeDOMReader attributeDOMReader;
    private final RootHandler rootHandler = new RootHandler();

    public CroisementDOMReader(CroisementEditor croisementEditor, MessageHandler messageHandler) {
        this.croisementEditor = croisementEditor;
        this.messageHandler = messageHandler;
        this.attributeDOMReader = new AttributeDOMReader(croisementEditor.getFichothequeEditor(), messageHandler);
    }

    public void readCroisement(Element element, SubsetItem subsetItem1, SubsetItem subsetItem2) {
        rootHandler.clear();
        DOMUtils.readChildren(element, rootHandler);
        rootHandler.flush(subsetItem1, subsetItem2);
    }


    private class RootHandler implements ElementHandler {

        private final Map<String, Lien> lienMap = new LinkedHashMap<String, Lien>();
        private final List<Element> attrTempList = new ArrayList<Element>();

        private RootHandler() {
        }

        private void clear() {
            lienMap.clear();
            attrTempList.clear();
        }

        private void flush(SubsetItem subsetItem1, SubsetItem subsetItem2) {
            List<Lien> lienList = CroisementUtils.wrap(lienMap.values().toArray(new Lien[lienMap.size()]));
            CroisementChange croisementChange = new InternalCroisementChange(lienList);
            Croisement croisement = croisementEditor.updateCroisement(subsetItem1, subsetItem2, croisementChange);
            if ((croisement != null) && (!attrTempList.isEmpty())) {
                for (Element element : attrTempList) {
                    attributeDOMReader.readAttrElement(element, croisement, element.getTagName());
                }
            }
            clear();
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lien")) {
                String mode = element.getAttribute("mode");
                int poids = getPoids(element);
                int position1 = getPosition(element, "position1");
                int position2 = getPosition(element, "position2");
                Lien lien = CroisementUtils.toLien(mode, poids, position1, position2);
                lienMap.put(mode, lien);
            } else if (tagName.equals("attr")) {
                attrTempList.add(element);
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

        private int getPoids(Element element) {
            int poids = 1;
            String poidsString = element.getAttribute("poids");
            if (poidsString.length() > 0) {
                try {
                    poids = Integer.parseInt(poidsString);
                    if (poids < 1) {
                        poids = 1;
                    }
                } catch (NumberFormatException nfe) {
                }
            }
            return poids;
        }

        private int getPosition(Element element, String attributeName) {
            int position = 0;
            String posString = element.getAttribute(attributeName);
            if (posString.length() > 0) {
                try {
                    position = Integer.parseInt(posString);
                    if (position < 0) {
                        position = 0;
                    }
                } catch (NumberFormatException nfe) {
                }
            }
            return position;
        }

    }


    private static class InternalCroisementChange implements CroisementChange {

        private final List<Lien> changedLienList;

        private InternalCroisementChange(List<Lien> changedLienList) {
            this.changedLienList = changedLienList;
        }

        @Override
        public List<String> getRemovedModeList() {
            return CroisementUtils.EMPTY_REMOVEDMODELIST;
        }

        @Override
        public List<Lien> getChangedLienList() {
            return changedLienList;
        }

    }

}
