/* FichothequeLib_Xml - Copyright (c) 2008-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import javax.xml.transform.Source;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.Lien;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.run.FicheExtractInfo;
import net.fichotheque.extraction.run.MotcleExtractInfo;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.conditions.TextTest;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.ListLangContext;
import net.mapeadores.util.localisation.UserLangContext;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLBuffer;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public final class ExtractionXMLUtils {

    private ExtractionXMLUtils() {
    }

    public static int getMainPoids(Croisement croisement) {
        if (croisement == null) {
            return 0;
        }
        return croisement.getLienList().get(0).getPoids();
    }

    public static void writeCroisement(XMLWriter xmlWriter, Croisement croisement) throws IOException {
        if (croisement == null) {
            return;
        }
        for (Lien lien : croisement.getLienList()) {
            xmlWriter.startOpenTag("lien");
            xmlWriter.addAttribute("mode", lien.getMode());
            xmlWriter.addAttribute("poids", lien.getPoids());
            xmlWriter.closeEmptyTag();
        }
    }

    public static void writeIntitule(XMLWriter xmlWriter, SubsetItem subsetItem, String phraseName, ExtractionContext extractionContext) throws IOException {
        xmlWriter.openTag("intitule");
        LangContext langContext = extractionContext.getLangContext();
        if (langContext instanceof ListLangContext) {
            for (ListLangContext.Unit unit : (ListLangContext) langContext) {
                Lang lang = unit.getLang();
                Locale locale = unit.getFormatLocale();
                String title = FichothequeUtils.getNumberPhrase(subsetItem, phraseName, lang, locale);
                LabelUtils.addLabel(xmlWriter, lang, title);
            }
        } else if (langContext instanceof UserLangContext) {
            UserLangContext userLangContext = (UserLangContext) langContext;
            Lang workingLang = userLangContext.getWorkingLang();
            Locale locale = userLangContext.getFormatLocale();
            String title = FichothequeUtils.getNumberPhrase(subsetItem, phraseName, workingLang, locale);
            LabelUtils.addLabel(xmlWriter, null, title);
        }
        xmlWriter.closeTag("intitule");
    }

    public static void writeFicheQuery(XMLWriter xmlWriter, FicheQuery ficheQuery, String tagName) throws IOException {
        xmlWriter.openTag(tagName);
        writeFicheQuery(xmlWriter, ficheQuery);
        xmlWriter.closeTag(tagName);
    }

    public static void writeFicheQuery(XMLWriter xmlWriter, FicheQuery ficheQuery) throws IOException {
        for (SubsetKey subsetKey : ficheQuery.getCorpusKeyList()) {
            xmlWriter.addSimpleElement("corpus", subsetKey.getSubsetName());
        }
        if (ficheQuery.isWithCurrentCorpus()) {
            xmlWriter.addEmptyElement("current");
        }
        Ranges idRanges = ficheQuery.getIdRanges();
        if (idRanges != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(idRanges));
        }
        String discardFilter = ficheQuery.getDiscardFilter();
        if (!discardFilter.equals(FicheQuery.ALL_DISCARDFILTER)) {
            xmlWriter.startOpenTag("discard");
            xmlWriter.addAttribute("filter", discardFilter);
            xmlWriter.closeEmptyTag();
        }
        writePeriodCondition(xmlWriter, ficheQuery.getPeriodCondition());
        FieldQuery fieldQuery = ficheQuery.getFieldQuery();
        if (fieldQuery != null) {
            short fieldConditionScope = fieldQuery.getScope();
            xmlWriter.startOpenTag("field-select");
            xmlWriter.addAttribute("scope", SelectionUtils.fieldConditionToString(fieldConditionScope));
            xmlWriter.endOpenTag();
            if (fieldConditionScope == FieldQuery.SELECTION_SCOPE) {
                for (FieldKey fieldKey : fieldQuery.getFieldKeyList()) {
                    xmlWriter.addSimpleElement("field", fieldKey.toString());
                }
            }
            ConditionsXMLStorage.appendCondition(xmlWriter, fieldQuery.getContentCondition());
            xmlWriter.closeTag("field-select");
        }
        String redacteurFilter = ficheQuery.getRedacteurFilter();
        if (redacteurFilter.equals(FicheQuery.SOME_REDACTEURFILTER)) {
            List<String> redacteurGlobalIdList = ficheQuery.getRedacteurGlobalIdList();
            List<SubsetKey> sphereKeyList = ficheQuery.getSphereKeyList();
            if ((!redacteurGlobalIdList.isEmpty()) || (!sphereKeyList.isEmpty())) {
                xmlWriter.openTag("redacteurs");
                for (String globalId : redacteurGlobalIdList) {
                    xmlWriter.startOpenTag("redacteur");
                    try {
                        xmlWriter.addAttribute("sphere", SphereUtils.getSubsetKey(globalId).getSubsetName());
                        xmlWriter.addAttribute("id", String.valueOf(SphereUtils.getId(globalId)));
                    } catch (ParseException pe) {
                    }
                    xmlWriter.closeEmptyTag();
                }
                for (SubsetKey subsetKey : sphereKeyList) {
                    xmlWriter.addSimpleElement("sphere", subsetKey.getSubsetName());
                }
                xmlWriter.closeTag("redacteurs");
            }
        } else if (!redacteurFilter.equals(FicheQuery.ALL_REDACTEURFILTER)) {
            xmlWriter.startOpenTag("redacteurs");
            xmlWriter.addAttribute("filter", redacteurFilter);
            xmlWriter.closeEmptyTag();
        }
        if (ficheQuery.isWithGeoloc()) {
            xmlWriter.addEmptyElement("geoloc");
        }
        MotcleCondition motcleCondition = ficheQuery.getMotcleCondition();
        if (motcleCondition != null) {
            List<MotcleQuery> indexationList = motcleCondition.getMotcleQueryList();
            if (indexationList.size() > 1) {
                xmlWriter.startOpenTag("motcle-select-mode");
                xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(motcleCondition.getSelectMode()));
                xmlWriter.closeEmptyTag();
            }
            for (MotcleQuery motcleQuery : indexationList) {
                xmlWriter.openTag("motcle-select");
                writeMotcleQuery(xmlWriter, motcleQuery);
                xmlWriter.closeTag("motcle-select");
            }
        }
        if (ficheQuery.isWithRattachement()) {
            xmlWriter.addEmptyElement("rattachement");
        }
        writeCroisementCondition(xmlWriter, ficheQuery.getCroisementCondition());
    }

    public static void writeMotcleQuery(XMLWriter xmlWriter, MotcleQuery motcleQuery, String tagName) throws IOException {
        xmlWriter.openTag(tagName);
        writeMotcleQuery(xmlWriter, motcleQuery);
        xmlWriter.closeTag(tagName);
    }

    public static void writeMotcleQuery(XMLWriter xmlWriter, MotcleQuery motcleQuery) throws IOException {
        for (SubsetKey subsetKey : motcleQuery.getThesaurusKeyList()) {
            xmlWriter.addSimpleElement("thesaurus", subsetKey.getSubsetName());
        }
        Ranges idRanges = motcleQuery.getIdRanges();
        if (idRanges != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(idRanges));
        }
        Ranges levelRanges = motcleQuery.getLevelRanges();
        if (levelRanges != null) {
            xmlWriter.addSimpleElement("level", RangeUtils.positiveRangetoString(levelRanges));
        }
        Condition contentCondition = motcleQuery.getContentCondition();
        if (contentCondition != null) {
            short scope = motcleQuery.getContentConditionScope();
            xmlWriter.startOpenTag("condition");
            xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(contentCondition.getMode()));
            if (scope != MotcleQuery.ONLY_IDALPHA_SCOPE) {
                xmlWriter.addAttribute("scope", SelectionUtils.motcleFieldConditionScopeToString(scope));
            }
            xmlWriter.endOpenTag();
            for (TextTest textTest : contentCondition.getIncludingTextTestList()) {
                ConditionsXMLStorage.appendTestText(textTest, xmlWriter);
            }
            for (TextTest textTest : contentCondition.getExcludingTextTestList()) {
                ConditionsXMLStorage.appendTestText(textTest, xmlWriter);
            }
            xmlWriter.closeTag("condition");
        }
        FicheCondition ficheCondition = motcleQuery.getFicheCondition();
        if (ficheCondition != null) {
            List<FicheQuery> indexationList = ficheCondition.getFicheQueryList();
            if (indexationList.size() > 1) {
                xmlWriter.startOpenTag("fiche-select-mode");
                xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(ficheCondition.getSelectMode()));
                xmlWriter.closeEmptyTag();
            }
            for (FicheQuery ficheQuery : indexationList) {
                xmlWriter.openTag("fiche-select");
                writeFicheQuery(xmlWriter, ficheQuery);
                xmlWriter.closeTag("fiche-select");
            }
        }
        if (motcleQuery.includeSatellites()) {
            xmlWriter.addEmptyElement("satellites");
        }
        writeCroisementCondition(xmlWriter, motcleQuery.getCroisementCondition());
    }

    public static void writeIllustrationQuery(XMLWriter xmlWriter, IllustrationQuery illustrationQuery, String tagName) throws IOException {
        if (!SelectionUtils.isEmpty(illustrationQuery)) {
            xmlWriter.openTag(tagName);
            writeIllustrationQuery(xmlWriter, illustrationQuery);
            xmlWriter.closeTag(tagName);
        }
    }

    public static void writeIllustrationQuery(XMLWriter xmlWriter, IllustrationQuery illustrationQuery) throws IOException {
        for (SubsetKey subsetKey : illustrationQuery.getAlbumKeyList()) {
            xmlWriter.addSimpleElement("album", subsetKey.getSubsetName());
        }
        writeCroisementCondition(xmlWriter, illustrationQuery.getCroisementCondition());
        for (String albumDimName : illustrationQuery.getAlbumDimNameList()) {
            xmlWriter.addSimpleElement("albumdim", albumDimName);
        }
    }

    public static void writeDocumentQuery(XMLWriter xmlWriter, DocumentQuery documentQuery, String tagName) throws IOException {
        xmlWriter.openTag(tagName);
        writeDocumentQuery(xmlWriter, documentQuery);
        xmlWriter.closeTag(tagName);
    }

    public static void writeDocumentQuery(XMLWriter xmlWriter, DocumentQuery documentQuery) throws IOException {
        for (SubsetKey subsetKey : documentQuery.getAddendaKeyList()) {
            xmlWriter.addSimpleElement("addenda", subsetKey.getSubsetName());
        }
        writeCroisementCondition(xmlWriter, documentQuery.getCroisementCondition());
    }

    private static void writeCroisementCondition(XMLWriter xmlWriter, CroisementCondition croisementCondition) throws IOException {
        if (croisementCondition != null) {
            String[] lienModeArray = croisementCondition.getLienModeArray();
            if (lienModeArray != null) {
                int length = lienModeArray.length;
                for (int i = 0; i < length; i++) {
                    String mode = lienModeArray[i];
                    if (mode.isEmpty()) {
                        mode = "_default";
                    }
                    xmlWriter.addSimpleElement("lien-mode", mode);
                }
            }
            Ranges poidsRanges = croisementCondition.getPoidsRanges();
            if (poidsRanges != null) {
                xmlWriter.startOpenTag("lien-poids");
                if (croisementCondition.isExclusionRangeList()) {
                    xmlWriter.addAttribute("mode", "exclude");
                }
                xmlWriter.endOpenTag();
                xmlWriter.addText(RangeUtils.positiveRangetoString(poidsRanges));
                xmlWriter.closeTag("lien-poids", false);
            }
        }
    }

    private static void writePeriodCondition(XMLWriter xmlWriter, PeriodCondition periodCondition) throws IOException {
        if (periodCondition == null) {
            return;
        }
        xmlWriter.startOpenTag("period");
        if (periodCondition.getStartType().equals(PeriodCondition.ANY_TYPE)) {
            xmlWriter.addAttribute("start", PeriodCondition.ANY_CHAR);
        } else {
            xmlWriter.addAttribute("start", periodCondition.getStartDate().toString());
        }
        switch (periodCondition.getEndType()) {
            case PeriodCondition.ANY_TYPE:
                xmlWriter.addAttribute("end", PeriodCondition.ANY_CHAR);
                break;
            case PeriodCondition.DATE_TYPE:
                xmlWriter.addAttribute("end", periodCondition.getEndDate().toString());
                break;
        }
        xmlWriter.endOpenTag();
        if (periodCondition.isOnCreationDate()) {
            xmlWriter.addSimpleElement("chrono", "creation");
        }
        if (periodCondition.isOnModificationDate()) {
            xmlWriter.addSimpleElement("chrono", "modification");
        }
        for (FieldKey fieldKey : periodCondition.getFieldKeyList()) {
            xmlWriter.addSimpleElement("field", fieldKey.toString());
        }
        xmlWriter.closeTag("period");
    }

    public static String ficheBlocksToExtractionString(FicheBlocks ficheBlocks, @Nullable ExtractionContext extractionContext, @Nullable SubsetKey defaultCorpusKey) {
        StringBuilder buf = new StringBuilder();
        AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(buf, -999);
        try {
            xmlWriter.openTag("section");
            if (ficheBlocks != null) {
                Corpus defaultCorpus = null;
                if ((defaultCorpusKey != null) && (extractionContext != null)) {
                    defaultCorpus = (Corpus) extractionContext.getFichotheque().getSubset(defaultCorpusKey);
                }
                FicheBlockXMLPart ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter);
                ficheBlockXMLPart.setCompleteMode(extractionContext);
                ficheBlockXMLPart.setDefaultCorpus(defaultCorpus);
                ficheBlockXMLPart.addFicheBlocks(ficheBlocks);
            }
            xmlWriter.closeTag("section");
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        return buf.toString();
    }

    public static Source ficheBlocksToSource(FicheBlocks ficheBlocks, @Nullable ExtractionContext extractionContext, @Nullable SubsetKey defaultSubsetKey) {
        XMLBuffer xmlBuffer = new XMLBuffer();
        XMLWriter xmlWriter = xmlBuffer.getXMLWriter();
        try {
            xmlWriter.openTag("section");
            if (ficheBlocks != null) {
                Corpus defaultCorpus = null;
                if ((defaultSubsetKey != null) && (extractionContext != null)) {
                    defaultCorpus = (Corpus) extractionContext.getFichotheque().getSubset(defaultSubsetKey);
                }
                FicheBlockXMLPart ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter);
                ficheBlockXMLPart.setCompleteMode(extractionContext);
                ficheBlockXMLPart.setDefaultCorpus(defaultCorpus);
                ficheBlockXMLPart.addFicheBlocks(ficheBlocks);
            }
            xmlWriter.closeTag("section");
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        xmlBuffer.flush();
        return xmlBuffer.toSAXSource();
    }

    public static MotcleExtractInfo toMotcleExtractInfo(Motcle motcle, MotcleFilter motcleFilter) {
        return new SimpleMotcleExtractInfo(motcle, motcleFilter);
    }

    public static FicheExtractInfo toFicheExtractInfo(FicheMeta ficheMeta, FicheFilter ficheFilter) {
        return new SimpleFicheExtractInfo(ficheMeta, ficheFilter);
    }

    public static FicheQuery derive(FicheQuery ficheQuery, Collection<SubsetKey> corpusKeys) {
        List<SubsetKey> templist = new ArrayList<SubsetKey>();
        for (SubsetKey subsetKey : corpusKeys) {
            if (subsetKey != null) {
                if (subsetKey.isCorpusSubset()) {
                    templist.add(subsetKey);
                }
            }
        }
        List<SubsetKey> list = FichothequeUtils.wrap(templist.toArray(new SubsetKey[templist.size()]));
        return new DerivationFicheQuery(ficheQuery, list);
    }

    public static MotcleQuery derive(MotcleQuery motcleQuery, Collection<SubsetKey> thesaurusKeys) {
        List<SubsetKey> templist = new ArrayList<SubsetKey>();
        for (SubsetKey subsetKey : thesaurusKeys) {
            if (subsetKey != null) {
                if (subsetKey.isThesaurusSubset()) {
                    templist.add(subsetKey);
                }
            }
        }
        List<SubsetKey> list = FichothequeUtils.wrap(templist.toArray(new SubsetKey[templist.size()]));
        return new DerivationMotcleQuery(motcleQuery, list);
    }

    public static short sortToShort(String sort) {
        if ((sort == null) || (sort.length() == 0)) {
            return ExtractionConstants.ASCENDING_SORT;
        }
        if (sort.equals("asc")) {
            return ExtractionConstants.ASCENDING_SORT;
        }
        if (sort.equals("desc")) {
            return ExtractionConstants.DESCENDING_SORT;
        }
        throw new IllegalArgumentException("unknwon sort : " + sort);
    }

    public static short typeToShort(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException("undefined type");
        }
        if (type.equals("poids")) {
            return ExtractionConstants.POIDS_TYPE;
        }
        if (type.equals("titre")) {
            return ExtractionConstants.TITRE_TYPE;
        }
        if (type.equals("annee")) {
            return ExtractionConstants.ANNEE_TYPE;
        }
        if (type.equals("lang")) {
            return ExtractionConstants.LANG_TYPE;
        }
        if (type.equals("pays")) {
            return ExtractionConstants.PAYS_TYPE;
        }
        if (type.equals("fields")) {
            return ExtractionConstants.FIELDS_TYPE;
        }
        if (type.equals("position")) {
            return ExtractionConstants.POSITION_TYPE;
        }
        throw new IllegalArgumentException("unknwon type : " + type);
    }

    public static String sortToString(short sort) {
        switch (sort) {
            case ExtractionConstants.ASCENDING_SORT:
                return "asc";
            case ExtractionConstants.DESCENDING_SORT:
                return "desc";
            default:
                throw new IllegalArgumentException("unknwon sort : " + sort);
        }
    }

    public static String typeToString(short type) {
        switch (type) {
            case ExtractionConstants.POIDS_TYPE:
                return "poids";
            case ExtractionConstants.TITRE_TYPE:
                return "titre";
            case ExtractionConstants.ANNEE_TYPE:
                return "annee";
            case ExtractionConstants.LANG_TYPE:
                return "lang";
            case ExtractionConstants.PAYS_TYPE:
                return "pays";
            case ExtractionConstants.FIELDS_TYPE:
                return "fields";
            case ExtractionConstants.POSITION_TYPE:
                return "position";
            default:
                throw new IllegalArgumentException("unknwon type : " + type);
        }
    }


    private static class SimpleMotcleExtractInfo implements MotcleExtractInfo {

        private final Motcle motcle;
        private final MotcleFilter motcleFilter;

        private SimpleMotcleExtractInfo(Motcle motcle, MotcleFilter motcleFilter) {
            this.motcle = motcle;
            this.motcleFilter = motcleFilter;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

        @Override
        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        @Override
        public Croisement getCroisement() {
            return null;
        }

    }


    private static class SimpleFicheExtractInfo implements FicheExtractInfo {

        private final FicheFilter ficheFilter;
        private final FicheMeta ficheMeta;

        private SimpleFicheExtractInfo(FicheMeta ficheMeta, FicheFilter ficheFilter) {
            this.ficheMeta = ficheMeta;
            this.ficheFilter = ficheFilter;
        }

        @Override
        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return ficheMeta;
        }

        @Override
        public Croisement getCroisement() {
            return null;
        }

        @Override
        public Object getGroupClauseObject() {
            return null;
        }

    }


    private static class DerivationFicheQuery implements FicheQuery {

        private final FicheQuery ficheQuery;
        private final List<SubsetKey> corpusKeyList;

        private DerivationFicheQuery(FicheQuery ficheQuery, List<SubsetKey> corpusKeyList) {
            this.ficheQuery = ficheQuery;
            this.corpusKeyList = corpusKeyList;
        }


        @Override
        public List<SubsetKey> getCorpusKeyList() {
            return corpusKeyList;
        }

        @Override
        public Ranges getIdRanges() {
            return ficheQuery.getIdRanges();
        }

        @Override
        public MotcleCondition getMotcleCondition() {
            return ficheQuery.getMotcleCondition();
        }

        @Override
        public FieldQuery getFieldQuery() {
            return ficheQuery.getFieldQuery();
        }

        @Override
        public String getRedacteurFilter() {
            return ficheQuery.getRedacteurFilter();
        }

        @Override
        public List<String> getRedacteurGlobalIdList() {
            return ficheQuery.getRedacteurGlobalIdList();
        }

        @Override
        public List<SubsetKey> getSphereKeyList() {
            return ficheQuery.getSphereKeyList();
        }


        @Override
        public CroisementCondition getCroisementCondition() {
            return ficheQuery.getCroisementCondition();
        }

        @Override
        public boolean isWithRattachement() {
            return ficheQuery.isWithRattachement();
        }

        @Override
        public PeriodCondition getPeriodCondition() {
            return ficheQuery.getPeriodCondition();
        }

        @Override
        public boolean isWithCurrentCorpus() {
            return ficheQuery.isWithCurrentCorpus();
        }

        @Override
        public String getDiscardFilter() {
            return ficheQuery.getDiscardFilter();
        }

        @Override
        public boolean isWithGeoloc() {
            return ficheQuery.isWithGeoloc();
        }


    }


    private static class DerivationMotcleQuery implements MotcleQuery {

        private final MotcleQuery motcleQuery;
        private final List<SubsetKey> thesaurusKeyList;

        private DerivationMotcleQuery(MotcleQuery motcleQuery, List<SubsetKey> thesaurusKeyList) {
            this.motcleQuery = motcleQuery;
            this.thesaurusKeyList = thesaurusKeyList;
        }

        @Override
        public Ranges getIdRanges() {
            return motcleQuery.getIdRanges();
        }

        @Override
        public Ranges getLevelRanges() {
            return motcleQuery.getLevelRanges();
        }

        @Override
        public List<SubsetKey> getThesaurusKeyList() {
            return thesaurusKeyList;
        }

        @Override
        public Condition getContentCondition() {
            return motcleQuery.getContentCondition();
        }

        @Override
        public short getContentConditionScope() {
            return motcleQuery.getContentConditionScope();
        }

        @Override
        public FicheCondition getFicheCondition() {
            return motcleQuery.getFicheCondition();
        }

        @Override
        public CroisementCondition getCroisementCondition() {
            return motcleQuery.getCroisementCondition();
        }

        @Override
        public boolean includeSatellites() {
            return motcleQuery.includeSatellites();
        }

    }

}
